/*
* bsp.h                                                     Version 5.2.0
*
* Board Support Package API Header for STMicro STM32 processors.
* All are similar so just use conditionals here for differences.
*
* Contains BSP-specific defines, types, prototypes, and configuration
* settings.
*
* See XBASE\bbsp.h.
*
* Copyright (c) 2002-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore, Xingsheng Wan
*
*****************************************************************************/

#ifndef SMX_BSP_H
#define SMX_BSP_H

#include "armdefs.h"  /* for processor defines */

#ifdef __cplusplus
extern "C" {
#endif

/* Configuration */

#if defined(SB_CPU_STM32F42X) || defined(SB_CPU_STM32F446) || defined(SB_CPU_STM32F46X)|| defined(SB_CPU_STM32F7XX) 
#define SB_USB_FS  0  /* Set to 1 if using USB full speed (on either USB controller) since PLL48 clock is used. */
#endif

#if defined(SB_CPU_STM32F101)
#define SB_CPU_HZ  36000000
#elif defined(SB_CPU_STM32F103)
#define SB_CPU_HZ  72000000
#elif defined(SB_CPU_STM32F107_105)
#define SB_CPU_HZ  72000000
#elif defined(SB_CPU_STM32F2XX)
#define SB_CPU_HZ 120000000
#elif defined(SB_CPU_STM32F30X)
#define SB_CPU_HZ  72000000
#elif defined(SB_CPU_STM32F37X)
#define SB_CPU_HZ  72000000
#elif defined(SB_CPU_STM32F411)
#define SB_CPU_HZ  96000000
#elif defined(SB_CPU_STM32F42X) || defined(SB_CPU_STM32F446) || defined(SB_CPU_STM32F46X)
 #if SB_USB_FS
#define SB_CPU_HZ 168000000
 #else
#define SB_CPU_HZ 180000000
 #endif
#elif defined(SB_CPU_STM32F4XX)
#define SB_CPU_HZ 168000000
#elif defined(SB_CPU_STM32F7XX)
 #if SB_USB_FS
#define SB_CPU_HZ 192000000
 #else
#define SB_CPU_HZ 200000000
 #endif
#elif defined(SB_CPU_STM32H723)
#define SB_CPU_HZ 550000000
#elif defined(SB_CPU_STM32H753)
#define SB_CPU_HZ 400000000
#elif defined(SB_CPU_STM32U575)
#define SB_CPU_HZ 160000000
#else
#error Define SB_CPU_HZ for your processor here in bsp.h.
#endif

/* SB_CPU_MHZ: 3 variants are provided for fractional clock rates.
   Use the one that rounds properly for each purpose.
*/
#define SB_CPU_MHZ_DOWN  ((SB_CPU_HZ) / 1000000)             /* round down */
#define SB_CPU_MHZ_RND   (((SB_CPU_HZ) + 500000) / 1000000)  /* round to nearest */
#define SB_CPU_MHZ_UP    (((SB_CPU_HZ) + 999999) / 1000000)  /* round up */

/* Console Configuration. See documentation in XBASE\bbsp.h. */

#define SB_CON_BAUD 115200

#if SB_CON_IN || SB_CON_OUT
#if defined(SB_BRD_STMICRO_STM3210BEVAL)      || defined(SB_BRD_STMICRO_STM3210EEVAL) || \
    defined(SB_BRD_STMICRO_STM32429IEVAL)     || defined(SB_BRD_EMCRAFT_STM32F439SK)  || \
    defined(SB_BRD_STMICRO_STM32446EEVAL)     || defined(SB_BRD_STMICRO_STM32469IEVAL) || \
    defined(SB_BRD_STMICRO_STM32746GEVAL)     || defined(SB_BRD_STMICRO_STM32F746G_DISCOVERY) || \
    defined(SB_BRD_EMCRAFT_STM32F746SK)       || \
    defined(SB_BRD_STMICRO_STM32F769IEVAL)    || defined(SB_BRD_STMICRO_STM32F779IEVAL) || \
    defined(SB_BRD_STMICRO_STM32H753IEVAL)    || defined(SB_BRD_STMICRO_STM32U575IEV)
#define SB_CON_IN_PORT       1  /* ports are numbered at 1 */
#define SB_CON_IN_USES_INT   0
#define SB_CON_OUT_PORT      1  /* ports are numbered at 1 */
#define SB_CON_OUT_USES_INT  0
#elif defined(SB_BRD_STMICRO_STM3210CEVAL) || defined(SB_BRD_STMICRO_STM32373CEVAL) || defined(SB_BRD_STMICRO_STM32F411E_DISCOVERY)
#define SB_CON_IN_PORT       2  /* ports are numbered at 1 */
#define SB_CON_IN_USES_INT   0
#define SB_CON_OUT_PORT      2  /* ports are numbered at 1 */
#define SB_CON_OUT_USES_INT  0
#elif defined(SB_BRD_STMICRO_STM3220GEVAL) || defined(SB_BRD_STMICRO_STM3240GEVAL) || defined(SB_BRD_STMICRO_STM32H723ZGNUCLEO)
#define SB_CON_IN_PORT       3  /* ports are numbered at 1 */
#define SB_CON_IN_USES_INT   0
#define SB_CON_OUT_PORT      3  /* ports are numbered at 1 */
#define SB_CON_OUT_USES_INT  0
#elif defined(SB_BRD_IAR_STM32F207ZGSK) || defined(SB_BRD_IAR_STM32F407ZGSK)
#define SB_CON_IN_PORT       6  /* ports are numbered at 1 */
#define SB_CON_IN_USES_INT   0
#define SB_CON_OUT_PORT      6  /* ports are numbered at 1 */
#define SB_CON_OUT_USES_INT  0
#else
#error Define SB_CON settings for your board here in bsp.h.
#endif
#endif // SB_CON_IN || SB_CON_OUT

#if defined(SB_CPU_STM32U5XX)
#define SB_HW_INIT_AT_MAIN       /* This BSP has init that must be called from main(),
                                    after compiler startup (static initializers, etc). */
#endif

#if defined(__IAR_SYSTEMS_ICC__)
#if defined(SB_BRD_STMICRO_STM32F746G_DISCOVERY) || \
    defined(SB_BRD_STMICRO_STM32746GEVAL) || defined(SB_BRD_STMICRO_STM32F769IEVAL) || \
    defined(SB_CPU_STM32H7XX) || defined(SB_CPU_STM32U5XX)
#define SB_ICF_NEW               /* This BSP uses our new style .icf files. */
#endif
#endif

#if defined(PEG) || defined(CPEG) || defined(SMX_TXPORT)
#define SB_DISP_DEMO       0     /* keep 0 */
#define SB_DISP_LCD        0     /* keep 0 */
#define SB_DISP_OLED       0     /* keep 0 */
#elif defined(SB_BRD_STMICRO_STM32F3DISCOVERY) || defined(SB_BRD_STMICRO_STM32F4DISCOVERY) || \
      defined(SB_BRD_STMICRO_STM32F411E_DISCOVERY) || defined(SB_BRD_STMICRO_STM32H723ZGNUCLEO) || \
      defined(SB_BRD_STMICRO_STM32H753IEVAL) || defined(SB_BRD_STMICRO_STM32U575IEV)
#define SB_DISP_DEMO       0     /* 1 to enable LCD demo. 0 to disable. */
#define SB_DISP_LCD        0     /* 1 if board has LCD display. 0 if not. */
#define SB_DISP_OLED       0     /* 1 if board has OLED display. 0 if not. */
#elif defined(SB_BRD_STMICRO_STM32) || defined(SB_BRD_IAR_STM32F207ZGSK) || defined(SB_BRD_IAR_STM32F407ZGSK)
#define SB_DISP_DEMO       0     /* 1 to enable LCD demo. 0 to disable. */
#define SB_DISP_LCD        0     /* 1 if board has LCD display. 0 if not. */
#define SB_DISP_OLED       0     /* 1 if board has OLED display. 0 if not. */
#else
#define SB_DISP_DEMO       0     /* 1 to enable LCD demo. 0 to disable. */
#define SB_DISP_LCD        0     /* 1 if board has LCD display. 0 if not. */
#define SB_DISP_OLED       0     /* 1 if board has OLED display. 0 if not. */
#endif

#if defined(SB_CPU_STM32F42X) || defined(SB_CPU_STM32F43X) || defined(SB_CPU_STM32F446) || \
        defined(SB_CPU_STM32F46X) || defined(SB_CPU_STM32F7XX)|| defined(SB_CPU_STM32H7XX) || \
        defined(SB_CPU_STM32U5XX)
#define SB_FPU_DEMO        0
#endif

/* Defines */

#define SB_INT_MIN         0
#define SB_INT_MAX         (SB_IRQ_MAX+16)
#define SB_INT_NUM         (SB_INT_MAX-SB_INT_MIN+1)

#define SB_IRQ_MIN         0     /* see sb_irq_table[] in irqtable.c */
#if defined(SB_CPU_STM32F107_105)
#define SB_IRQ_MAX         67
#elif defined(SB_CPU_STM32F2XX)
#define SB_IRQ_MAX         80
#elif defined(SB_CPU_STM32F30X)
#define SB_IRQ_MAX         81
#elif defined(SB_CPU_STM32F37X)
#define SB_IRQ_MAX         81
#elif defined(SB_CPU_STM32F42X) || defined(SB_CPU_STM32F43X)
#define SB_IRQ_MAX         90
#elif defined(SB_CPU_STM32F446)
#define SB_IRQ_MAX         96
#elif defined(SB_CPU_STM32F46X)
#define SB_IRQ_MAX         92
#elif defined(SB_CPU_STM32F411)
#define SB_IRQ_MAX         85
#elif defined(SB_CPU_STM32F4XX)
#define SB_IRQ_MAX         81
#elif defined(SB_CPU_STM32F74X) || defined(SB_CPU_STM32F75X)
#define SB_IRQ_MAX         97
#elif defined(SB_CPU_STM32F76X) || defined(SB_CPU_STM32F77X)
#define SB_IRQ_MAX         109
#elif defined(SB_CPU_STM32H723) || defined(SB_CPU_STM32H733)
#define SB_IRQ_MAX         162
#elif defined(SB_CPU_STM32H743) || defined(SB_CPU_STM32H753)
#define SB_IRQ_MAX         149
#elif defined(SB_CPU_STM32U5XX)
#define SB_IRQ_MAX         124
#else
#define SB_IRQ_MAX         59
#endif
#define SB_IRQ_NUM         (SB_IRQ_MAX-SB_IRQ_MIN+1)
#define SB_IRQ_PRI_MIN     INTC_VIC_MIN_PRIORITY
#define SB_IRQ_PRI_MAX     INTC_VIC_MAX_PRIORITY

#define SB_IRQ_PRI_BITS    0xF0  /* only 4 bits 7:4 are used for priority */
#define SB_IRQ_PRI_INCR    0x10  /* priority increment */

/* Max number of interrupt mask registers */
#define SB_IRQ_MASK_REGS ((SB_IRQ_MAX + 1 + 31) / 32)

#define SB_TICK_IRQ        -1    /* Dummy value. smx tick uses SysTick which is not in irq_table[]. */

#define SB_TICK_TMR_COUNTS_PER_TICK ((SB_CPU_HZ)/(SMX_CFG_TICKS_PER_SEC))

#if defined(__IAR_SYSTEMS_ICC__) && defined(SB_ICF_NEW)
#pragma section = "EVT" /* defined in .icf file */
#else
#define SB_EVT_RAM_BASE    0x20000000  /* Alignment matters. Simplest to put at start of SRAM. */
#endif


/* Typedefs */

typedef struct
{
   u8  pri;            /* interrupt priority */
   /* no need for vector number since easy to calculate from IRQ */
} SB_IRQ_REC;

typedef union { ISR_PTR __fun; void * __ptr; } intvec_elem;


/* Macros */


/* Platform-Specific Functions */

BOOLEAN sb_IRQTableEntryWrite(int irq_num, int pri);


/* Local Functions */


#ifdef __cplusplus
}
#endif

#endif /* SMX_BSP_H */
