/*
* bcfg.h                                                    Version 5.2.0
*
* smxBase Configuration.
*
* Copyright (c) 2010-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: David Moore and Yingbo Hu
*
*****************************************************************************/

#ifndef SB_BCFG_H
#define SB_BCFG_H

#if defined(_ARM) || defined(arm) || defined(__arm__) || defined(__ICCARM__)
#define SB_CPU_ARM
#else
/* Use what was passed on the command line. */
#endif

#if defined(SB_CPU_ARM) && \
    (defined(__TARGET_PROFILE_M)) ||             /* ARM RealView/Keil */ \
    (defined(__ARM_ARCH_7M__))    ||             /* GNU */ \
    (defined(__CORE__) &&                        /* IAR */ \
     ((__CORE__ == __ARM7M__) || (__CORE__ == __ARM7EM__) || \
      (__CORE__ == __ARM8M_BASELINE__) || (__CORE__ == __ARM8M_MAINLINE__)))

#define SB_CPU_ARMM  /* Remember ARM is also defined since compiler defines it or variant */

#if defined(__CORE__) && ((__CORE__ == __ARM8M_BASELINE__) || (__CORE__ == __ARM8M_MAINLINE__))
#define SB_CPU_ARMM7 0
#define SB_CPU_ARMM8 1
#else
#define SB_CPU_ARMM7 1
#define SB_CPU_ARMM8 0
#endif
#endif /* SB_CPU_ARM */

#define SB_CPU_32BIT
#define SB_CPU_MEM_MAPPED_IO  1
#define SB_CFG_IRQ_MAX_NUM    10  /* Max num IRQs used by app + middleware. Also change bcfg.inc, if present. */
#define SB_CFG_SHORT_ENUMS    1
#define SB_CFG_INT64_TYPE     1

#if defined(__IAR_SYSTEMS_ICC__) || defined(__MWERKS__)
#define SB_CFG_EXACT_WIDTH_TYPES 1
#else
#define SB_CFG_EXACT_WIDTH_TYPES 0
#endif

#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199409L)
#define SB_CFG_STDBOOL 1
#else
#define SB_CFG_STDBOOL 0
#endif

#if defined(SMX_TSMX)
#define SB_CFG_UARTI 0
#elif defined(SB_CPU_STM32) || defined(SB_CPU_KXX) || defined(SB_CPU_AT91SAMV)
#define SB_CFG_UARTI 1
#else
#define SB_CFG_UARTI 0
#endif

#define SB_CON_IN  1
#define SB_CON_OUT 1
#define SB_CFG_OMB_SIZE    512  /* Output message buffer size (bytes) */
#define SB_CFG_OMQ_SIZE     40  /* Output message queue size (records) */

#if SB_CFG_UARTI
/* Buffer Sizes for Interrupt-Driven UART Driver for Terminal I/O */
#if SB_CON_IN
#define SB_CON_IN_BUFSZ    16
#else
#define SB_CON_IN_BUFSZ     0
#endif
#if SB_CON_OUT
#define SB_CON_OUT_BUFSZ  256
#else
#define SB_CON_OUT_BUFSZ    0
#endif
#endif

/* Enable time measurements */
#define SB_CFG_TM 1             /* Also set in xarmm_iar.inc */

#endif /* SB_BCFG_H */

