/*
* buart.h                                                   Version 5.2.0
*
* UART driver (interrupt-driven) API and high level definitions.
*
* Copyright (c) 2012-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Yingbo Hu, David Moore, Ralph Moore
*
*****************************************************************************/

#ifndef SB_BUART_H
#define SB_BUART_H

#include "smxbase.h"

/* Parity */
#define SUR_PARITY_NONE                 0
#define SUR_PARITY_ODD                  1
#define SUR_PARITY_EVEN                 2
#define SUR_PARITY_MARK                 3
#define SUR_PARITY_SPACE                4

/* Line Coding for Stop Bits */
#define SUR_STOP_BITS_1                 0
#define SUR_STOP_BITS_1_5               1
#define SUR_STOP_BITS_2                 2

/* Flow Control Flags */
#define SUR_FC_NONE                     0
#define SUR_FC_HARDWARE                 1
#define SUR_FC_XONXOFF                  2

typedef void (* SUR_CALLBACK)(uint iParameter);


/* Application (High Level) Interface */
int  sur_Open(uint port, u32 baudrate, uint parity, uint dbit, uint sbit, uint ibs, uint obs, uint fc);
int  sur_Close(uint port);
uint sur_GetIRQ(uint port);
int  sur_InByte(uint port, u8* pb, SUR_CALLBACK cb, uint tmo);
int  sur_InData(uint port, u8* pdst, uint len, u8 term, SUR_CALLBACK cb, uint tmo);
int  sur_OutByte(uint port, u8 b, SUR_CALLBACK cb, uint tmo);
int  sur_OutData(uint port, u8* psrc, uint len, u8 term, SUR_CALLBACK cb, uint tmo);

#endif /* SB_BUART_H */
