/*
* buarthw.h                                                 Version 5.2.0
*
* UART driver low level hardware interface definitions.
*
* Copyright (c) 2012-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Yingbo Hu
*
*****************************************************************************/

#ifndef SB_BUARTHW_H
#define SB_BUARTHW_H

#if SB_CFG_UARTI

#define SB_UART_MAX_NUM 2

/* Ring buffer handle for send/received data */
typedef struct
{
    uint  uSize;
    u8*   pBuf;
    u8*   pRead;
    u8*   pWrite;
}SUR_RINGBUF_T;

typedef struct
{
    uint             port;
    int              IRQ;
    SUR_RINGBUF_T*   pInBuf;
    SUR_RINGBUF_T*   pOutBuf;
    SUR_CALLBACK     pInCallback;
    SUR_CALLBACK     pOutCallback;
    uint             InExpected;
    uint             InTerm;
    uint             OutExpected;
    uint             flag;
    uint             flowctrl;
    BOOLEAN          InPending;
    BOOLEAN          OutPending;
    SCB_PTR          pInSem;
    SCB_PTR          pOutSem;
    MUCB_PTR         pInMutex;
    MUCB_PTR         pOutMutex;
}SUR_HANDLE_T;

int UARTInterruptHandler(uint iParameter);

/* Hardware interface functions (Low Level). Should be implemented in uarti.c in each BSP directory. */
void sur_HWInit(uint port, u32 baudrate, uint parity, uint dbit, uint sbit, SUR_HANDLE_T* phdl);
void sur_HWRelease(uint port);
void sur_HWStartXmit(uint port, SUR_HANDLE_T* phdl);

/* Utility functions used by Low Level driver */
uint sur_InAvail(SUR_HANDLE_T* phdl);
uint sur_InFreeSize(SUR_HANDLE_T* phdl);
uint sur_InWriteByte(SUR_HANDLE_T* phdl, u8 b);
uint sur_OutAvail(SUR_HANDLE_T* phdl);
uint sur_OutFreeSize(SUR_HANDLE_T* phdl);
uint sur_OutReadByte(SUR_HANDLE_T* phdl, u8* pb);

#endif /* SB_CFG_UARTI */
#endif /* SB_BUART_H */
