/*
* xevb.h                                                    Version 5.2.0
*
* smx Event Buffer macros and defines.
*
* Copyright (c) 2002-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Marty Cochran, David Moore, Ralph Moore
*
*****************************************************************************/

#ifndef SMX_XEVB_H
#define SMX_XEVB_H

/* Log Enable Flags. Enable logging of indicated events. */
#define SMX_EVB_EN_TASK       0x00000001  /* log task events */
#define SMX_EVB_EN_LSR        0x00000004  /* log LSR events */
#define SMX_EVB_EN_ISR        0x00000008  /* log ISR events */
#define SMX_EVB_EN_ERR        0x00000010  /* log errors */
#define SMX_EVB_EN_USER       0x00000020  /* log user events */
#define SMX_EVB_EN_PORTAL     0x00000040  /* log portal events */
#define SMX_EVB_EN_PERR       0x00000080  /* log portal errors */
#define SMX_EVB_EN_SSR1       0x00010000  /* log SSR group 1 (see SSR IDs in xdef.h) */
#define SMX_EVB_EN_SSR2       0x00020000  /* log SSR group 2 */
#define SMX_EVB_EN_SSR3       0x00040000  /* log SSR group 3 */
#define SMX_EVB_EN_SSR4       0x00080000  /* log SSR group 4 */
#define SMX_EVB_EN_SSR5       0x00100000  /* log SSR group 5 */
#define SMX_EVB_EN_SSR6       0x00200000  /* log SSR group 6 */
#define SMX_EVB_EN_SSR7       0x00400000  /* log SSR group 7 */
#define SMX_EVB_EN_SSR8       0x00800000  /* log SSR group 8 */
#define SMX_EVB_EN_SSRS       0x00FF0000  /* log SSR groups 1-8 */
#define SMX_EVB_EN_ALL        0x00FF00FD  /* log all events */

/* Record Types */
#define SMX_EVB_RT_ERR        0x00000100
#define SMX_EVB_RT_INVOKE     0x00000200
#define SMX_EVB_RT_ISR        0x00000300
#define SMX_EVB_RT_ISR_RET    0x00000400
#define SMX_EVB_RT_LSR        0x00000500
#define SMX_EVB_RT_LSR_RET    0x00000600
#define SMX_EVB_RT_SSR        0x00000700
#define SMX_EVB_RT_SSR_RET    0x00000800
#define SMX_EVB_RT_START      0x00000900
#define SMX_EVB_RT_RESUME     0x00000A00
#define SMX_EVB_RT_AUTOSTOP   0x00000B00
#define SMX_EVB_RT_PERR       0x00000C00
#define SMX_EVB_RT_USER       0x00001000
#define SMX_EVB_RT_PRINT      0x00001100
#define SMX_EVB_RT_PORTAL     0x00001200
#define SMX_EVB_RT_PORTAL_RET 0x00001300
#define SMX_EVB_RT_MASK       0x0000FF00

#define SMX_EVB_MAX_REC       11          /* maximum record size (words) */

typedef enum {
   STARTED = 8,
   START_PREEMPT,
   START_SUSPEND,
   START_STOP = 12,
   RESUMED = 16,
   RESUME_PREEMPT,
   RESUME_SUSPEND,
   RESUME_STOP = 20
} BECODE;

#ifdef __cplusplus
extern "C" {
#endif

BOOLEAN smx_EVBInit(u32 flags);

/* For system events */
void smx_EVBLogISR(void* isr);
void smx_EVBLogISRRet(void* isr);
void smx_EVBLogLSR(void* lsr);
void smx_EVBLogLSRRet(void* lsr);
void smx_EVBLogError(u32 errnum, void* h);
void smx_EVBLogInvoke(void* isr, LCB_PTR lsr, u32 par);
void smx_EVBLogSSRRet(u32 rv, u32 id);
void smx_EVBLogTaskAutoStop(void);
void smx_EVBLogTaskEnd(void);
void smx_EVBLogTaskResume(void);
void smx_EVBLogTaskStart(void);

/* For use from smx_SSR_ENTERn() macros */
void smx_EVBLogSSR0(u32 id);
void smx_EVBLogSSR1(u32 id, u32 p1);
void smx_EVBLogSSR2(u32 id, u32 p1, u32 p2);
void smx_EVBLogSSR3(u32 id, u32 p1, u32 p2, u32 p3);
void smx_EVBLogSSR4(u32 id, u32 p1, u32 p2, u32 p3, u32 p4);
void smx_EVBLogSSR5(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5);
void smx_EVBLogSSR6(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6);
void smx_EVBLogSSR7(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6, u32 p7);

/* For user events */
void smx_EVBLogUser0(void* h);
void smx_EVBLogUser1(void* h, u32 p1);
void smx_EVBLogUser2(void* h, u32 p1, u32 p2);
void smx_EVBLogUser3(void* h, u32 p1, u32 p2, u32 p3);
void smx_EVBLogUser4(void* h, u32 p1, u32 p2, u32 p3, u32 p4);
void smx_EVBLogUser5(void* h, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5);
void smx_EVBLogUser6(void* h, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6);
void smx_EVBLogUserPrint(u32 time, u32 index);

#ifdef __cplusplus
}
#endif


/*===========================================================================*
*                            EVENT BUFFER MACROS                             *
*===========================================================================*/

#if SMX_CFG_EVB
/*
   Format of first word of EVB records:

      0x5555TTSS

      5555 = Recognition pattern
      T = Type
      S = Size (Words)
*/

#if SMX_CFG_MACROS   /* use log macros */

/* Put at the start of ISRs. <1> */
#define smx_EVB_LOG_ISR(isr) \
   { \
      if (smx_evben & SMX_EVB_EN_ISR) \
      { \
         u32 istate, *p; \
         istate = sb_IntStateSaveDisable(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550004 | SMX_EVB_RT_ISR; \
         *p++ = (u32)smx_ct; \
         *p++ = (u32)(isr); \
         *p++ = sb_PtimeGet(); \
         smx_evbn = p; \
         sb_IntStateRestore(istate); \
      } \
   }

/* Put at the end of ISRs. <1> */
#define smx_EVB_LOG_ISR_RET(isr) \
   { \
      if (smx_evben & SMX_EVB_EN_ISR) \
      { \
         u32 istate, *p; \
         istate = sb_IntStateSaveDisable(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550003 | SMX_EVB_RT_ISR_RET; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)(isr); \
         smx_evbn = p; \
         sb_IntStateRestore(istate); \
      } \
   }

/* Put at the start of LSRs. <1> */
#define smx_EVB_LOG_LSR(lsr) \
   { \
      if (smx_evben & SMX_EVB_EN_LSR && !(lsr)->flags.nolog) \
      { \
         u32 istate, *p; \
         istate = sb_IntStateSaveDisable(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550004 | SMX_EVB_RT_LSR; \
         *p++ = (u32)smx_ct; \
         *p++ = (u32)(lsr); \
         *p++ = sb_PtimeGet(); \
         smx_evbn = p; \
         sb_IntStateRestore(istate); \
      } \
   }

/* Put at the end of LSRs. <1> */
#define smx_EVB_LOG_LSR_RET(lsr) \
   { \
      if (smx_evben & SMX_EVB_EN_LSR && !(lsr)->flags.nolog) \
      { \
         u32 istate, *p; \
         istate = sb_IntStateSaveDisable(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550003 | SMX_EVB_RT_LSR_RET; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)(lsr); \
         smx_evbn = p; \
         sb_IntStateRestore(istate); \
      } \
   }

/* Used in smx_EM() (xem.c). <1> */
#define smx_EVB_LOG_ERROR(errnum, hndl) \
   { \
      if (smx_evben & SMX_EVB_EN_ERR) \
      { \
         u32 istate, *p; \
         istate = sb_IntStateSaveDisable(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550005 | SMX_EVB_RT_ERR; \
         *p++ = sb_PtimeGet(); \
         *p++ = smx_etime; \
         *p++ = (u32)(errnum); \
         *p++ = (u32)(hndl); \
         smx_evbn = p; \
         sb_IntStateRestore(istate); \
      } \
   }

/* Used in smx_LSR_INVOKE() and smx_LSRInvoke(). Assumes interrupts are disabled
   by invoke macro or SSR. */
#define smx_EVB_LOG_INVOKE(isr, lsr, par) \
   { \
      if (smx_evben & SMX_EVB_EN_LSR) \
      { \
         u32 *p; \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550006 | SMX_EVB_RT_INVOKE; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)smx_ct; \
         *p++ = (u32)(isr); \
         *p++ = (u32)(lsr); \
         *p++ = (par); \
         smx_evbn = p; \
      } \
   }

/* Put at the end of SSRs. Assumes interrupts are enabled */
#define smx_EVB_LOG_SSR_RET(rv, id) \
   { \
      if (smx_evben & (id & SMX_ID_MASK_SSRGRP)) \
      { \
         u32 *p; \
         sb_INT_DISABLE(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550005 | SMX_EVB_RT_SSR_RET; \
         *p++ = sb_PtimeGet(); \
         *p++ = (smx_clsr != NULL) ? (u32)smx_clsr : (u32)smx_ct; \
         *p++ = smx_sched; \
         *p++ = (rv); \
         smx_evbn = p; \
         sb_INT_ENABLE(); \
      } \
   }

/* Task suspended or stopped. smx_sched_save used by some EVB_LOG_TASK macros. */
#define smx_EVB_LOG_TASK_END()   smx_sched_save = smx_sched;

/* Log autostop. Assumes interrupts are enabled. */
#define smx_EVB_LOG_TASK_AUTOSTOP() \
   { \
      if (smx_evben & SMX_EVB_EN_TASK) \
      { \
         u32 *p; \
         sb_INT_DISABLE(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550003 | SMX_EVB_RT_AUTOSTOP; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)smx_ct; \
         smx_evbn = p; \
         sb_INT_ENABLE(); \
      } \
   }

/* Log task resume. Assumes interrupts are enabled. */
#define smx_EVB_LOG_TASK_RESUME() \
   { \
      if (smx_evben & SMX_EVB_EN_TASK) \
      { \
         u32 *p; \
         sb_INT_DISABLE(); \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550004 | SMX_EVB_RT_RESUME; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)smx_ct; \
         *p++ = (BECODE)(RESUMED | smx_sched_save); \
         smx_evbn = p; \
         sb_INT_ENABLE(); \
      } \
   }

/* Log task start. Assumes interrupts are disabled. */
#define smx_EVB_LOG_TASK_START() \
   { \
      if (smx_evben & SMX_EVB_EN_TASK) \
      { \
         u32 *p; \
         p = (smx_evbn > smx_evbx) ? smx_evbi : smx_evbn; \
         *p++ = 0x55550004 | SMX_EVB_RT_START; \
         *p++ = sb_PtimeGet(); \
         *p++ = (u32)smx_ct; \
         *p++ = (BECODE)(STARTED | smx_sched_save); \
         smx_evbn = p; \
      } \
   }

#else    /* use log functions */

#define smx_EVB_LOG_ISR(isr)              smx_EVBLogISR(isr)
#define smx_EVB_LOG_ISR_RET(isr)          smx_EVBLogISRRet(isr)
#define smx_EVB_LOG_LSR(lsr)              smx_EVBLogLSR(lsr)
#define smx_EVB_LOG_LSR_RET(lsr)          smx_EVBLogLSRRet(lsr)
#define smx_EVB_LOG_ERROR(errnum, hndl)   smx_EVBLogError(errnum, hndl)
#define smx_EVB_LOG_INVOKE(isr, lsr, par) smx_EVBLogInvoke(isr, lsr, par)
#define smx_EVB_LOG_SSR_RET(rv, id)       smx_EVBLogSSRRet(rv, id)
#define smx_EVB_LOG_TASK_AUTOSTOP()       smx_EVBLogTaskAutoStop()
#define smx_EVB_LOG_TASK_END()            smx_EVBLogTaskEnd()
#define smx_EVB_LOG_TASK_RESUME()         smx_EVBLogTaskResume()
#define smx_EVB_LOG_TASK_START()          smx_EVBLogTaskStart()

#endif   /* end SMX_CFG_MACROS */

#define smx_EVB_LOG_USER0(h)  smx_EVBLogUser0(h);
#define smx_EVB_LOG_USER1(h, p1)  smx_EVBLogUser1(h, p1);
#define smx_EVB_LOG_USER2(h, p1, p2)  smx_EVBLogUser2(h, p1, p2);
#define smx_EVB_LOG_USER3(h, p1, p2, p3)  smx_EVBLogUser3(h, p1, p2, p3);
#define smx_EVB_LOG_USER4(h, p1, p2, p3, p4)  smx_EVBLogUser4(h, p1, p2, p3, p4);
#define smx_EVB_LOG_USER5(h, p1, p2, p3, p4, p5)  smx_EVBLogUser5(h, p1, p2, p3, p4, p5);
#define smx_EVB_LOG_USER6(h, p1, p2, p3, p4, p5, p6)  smx_EVBLogUser6(h, p1, p2, p3, p4, p5, p6);

#define smx_EVB_LOG_USER_PRINT(time, index)  smx_EVBLogUserPrint(time, index);

#else  /* event logging disabled */

#define smx_EVB_LOG_ISR(isr)
#define smx_EVB_LOG_ISR_RET(isr)
#define smx_EVB_LOG_LSR(lsr)
#define smx_EVB_LOG_LSR_RET(lsr)
#define smx_EVB_LOG_USER0(h)
#define smx_EVB_LOG_USER1(h, p1)
#define smx_EVB_LOG_USER2(h, p1, p2)
#define smx_EVB_LOG_USER3(h, p1, p2, p3)
#define smx_EVB_LOG_USER4(h, p1, p2, p3, p4)
#define smx_EVB_LOG_USER5(h, p1, p2, p3, p4, p5)
#define smx_EVB_LOG_USER6(h, p1, p2, p3, p4, p5, p6)

#define smx_EVB_LOG_ERROR(errnum, hndl)
#define smx_EVB_LOG_INVOKE(isr, lsr, par)
#define smx_EVB_LOG_SSR_RET(rv, id)
#define smx_EVB_LOG_TASK_AUTOSTOP()
#define smx_EVB_LOG_TASK_END()
#define smx_EVB_LOG_TASK_RESUME()
#define smx_EVB_LOG_TASK_START()
#define smx_EVB_LOG_USER_PRINT(time, index)

#endif  /* SMX_CFG_EVB */

/*
   Notes:

   1. sb_IntStateSaveDisable() and sb_IntStateRestore(istate) are used
      to save and restore the interrupt state in macros where interrupts
      may or may not have been enabled (or fully enabled, in the case of
      processors with an interrupt level such as ColdFire).
      smx_EVB_LOG_ERROR() needs to do this too since an error might be
      reported from an ISR, such as LQ_OVFL. Others can use sb_INT_DISABLE()
      and sb_INT_ENABLE() since interrupts are known to have been fully
      enabled.
*/

#endif  /* SMX_XEVB_H */

