/*
* bkbd.h                                                    Version 4.0.1
*
* Defines for Keyboard Handler (or terminal).
*
* Copyright (c) 1999-2019 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#ifndef SB_BKBD_H
#define SB_BKBD_H

/* Keyboard Flags */
/*
   The flags below are defined for comparison to the whole u32 key code,
   not just the shift status byte. The format of the u32 key code is shown
   at the top of x86 kbd.c.
*/

/* Shift status byte flags (3rd byte) (same as bios_keybrd(), etc, << 16). */
/* Do not change values or LED-setting code in x86 kbd.c may need to be changed. */
/* These are also used by the smxUSBH keyboard driver. */

#define SB_KBD_FL_RSHIFT  0x00010000L
#define SB_KBD_FL_LSHIFT  0x00020000L
#define SB_KBD_FL_CTRL    0x00040000L
#define SB_KBD_FL_ALT     0x00080000L
#define SB_KBD_FL_SCROLL  0x00100000L
#define SB_KBD_FL_NUM     0x00200000L
#define SB_KBD_FL_CAPS    0x00400000L
#define SB_KBD_FL_INS     0x00800000L

/* Other flags (4th byte) */
/* (none) */


#ifdef __cplusplus
extern "C" {
#endif

/* Function Prototypes */

#if (SB_CON_IN)
void sb_KbdInit(void);
void sb_KbdExit(void);
char sb_KbdConvertKey(u32 key32);
char sb_KbdGetCharPoll(void);
void sb_KbdStoreKeyLSR(u32 key32);
#else
#define sb_KbdInit()
#define sb_KbdExit()
#define sb_KbdConvertKey(key32)
#define sb_KbdGetCharPoll()
#define sb_KbdStoreKeyLSR(key32)
#endif

#ifdef __cplusplus
}
#endif

#endif /* SB_BKBD_H */

