/*
* xapiu.h                                                   Version 5.2.0
*
* smx API functions for umode, only
*
* Copyright (c) 1989-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore
*
*****************************************************************************/

#ifndef SMX_XAPIU_H
#define SMX_XAPIU_H

#if SMX_CFG_MPU

#ifndef SMX_XAPIU1_H /* xapip.h undefs SMX_XAPIU_H but not this, */
#define SMX_XAPIU1_H /* so prototypes only appear once, else default par redefs give error */

/*===========================================================================*
*                             smxu Kernel API                                *
*===========================================================================*/

#ifdef __cplusplus
extern "C" {
#endif

BCB_PTR     smxu_BlockGet(PCB_PTR pool, u8** bpp, u32 clrsz=0, BCB_PTR* bhp=NULL);
BCB_PTR     smxu_BlockMake(PCB_PTR pool, u8* bp, BCB_PTR* bhp=NULL);
u32         smxu_BlockPeek(BCB_PTR blk, SMX_PK_PAR par);
BOOLEAN     smxu_BlockRel(BCB_PTR blk, u16 clrsz=0);
u32         smxu_BlockRelAll(TCB_PTR task);
u8*         smxu_BlockUnmake(PCB_PTR* pool, BCB_PTR blk);

PCB_PTR     smxu_BlockPoolCreate(u8* p, u16 num, u16 size, const char* name=NULL, PCB_PTR* php=NULL);
u8*         smxu_BlockPoolDelete(PCB_PTR* php);
u32         smxu_BlockPoolPeek(PCB_PTR pool, SMX_PK_PAR par);

BOOLEAN     smxu_EventFlagsPulse(EGCB_PTR eg, u16 pulse_mask);
BOOLEAN     smxu_EventFlagsSet(EGCB_PTR eg, u16 set_mask, u16 pre_clear_mask);
u32         smxu_EventFlagsTest(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout=SMX_TMO_DFLT);
void        smxu_EventFlagsTestStop(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout=SMX_TMO_DFLT);

BOOLEAN     smxu_EventGroupClear(EGCB_PTR eg, u16 init_mask);
EGCB_PTR    smxu_EventGroupCreate(u32 init_mask, const char* name=NULL, EGCB_PTR* eghp=NULL);
BOOLEAN     smxu_EventGroupDelete(EGCB_PTR* eghp);
u32         smxu_EventGroupPeek(EGCB_PTR eg, SMX_PK_PAR par);

BOOLEAN     smxu_EventQueueClear(EQCB_PTR eq);
BOOLEAN     smxu_EventQueueCount(EQCB_PTR eq, u32 count, u32 timeout=SMX_TMO_DFLT);
void        smxu_EventQueueCountStop(EQCB_PTR eq, u32 count, u32 timeout=SMX_TMO_DFLT);
EQCB_PTR    smxu_EventQueueCreate(const char* name=NULL, EQCB_PTR* eqhp=NULL);
BOOLEAN     smxu_EventQueueDelete(EQCB_PTR* eqhp);
BOOLEAN     smxu_EventQueueSignal(EQCB_PTR eq);

u32         smxu_HeapBinPeek(u32 binno, EH_PK_PAR par, u32 hn=0);
BOOLEAN     smxu_HeapBinScan(u32 binno, u32 fnum, u32 bnum, u32 hn=0);
BOOLEAN     smxu_HeapBinSort(u32 binno, u32 fnum, u32 hn=0);
void*       smxu_HeapCalloc(u32 num, u32 sz, u32 an=0, u32 hn=0);
u32         smxu_HeapChunkPeek(void* vp, EH_PK_PAR par, u32 hn=0);
BOOLEAN     smxu_HeapFree(void* bp, u32 hn=0);
void*       smxu_HeapMalloc(u32 sz, u32 an=0, u32 hn=0);
u32         smxu_HeapPeek(EH_PK_PAR par, u32 hn=0);
void*       smxu_HeapRealloc(void* cbp, u32 sz, u32 an=0, u32 hn=0);

BOOLEAN     smxu_HTAdd(void* h, const char* name);
BOOLEAN     smxu_HTDelete(void* h);
void*       smxu_HTGetHandle(const char* name);
const char* smxu_HTGetName(void* h);

LCB_PTR     smxu_LSRCreate(FUN_PTR fun, TCB_PTR htask, u32 ssz, u32 flags, const char* name, LCB_PTR* lhp=NULL);
BOOLEAN     smxu_LSRDelete(LCB_PTR* lhp);
BOOLEAN     smxu_LSRInvoke(LCB_PTR lsr, u32 par=0);
void        smxu_LSRsOff(void);
BOOLEAN     smxu_LSRsOn(void);

BOOLEAN     smxu_MsgBump(MCB_PTR msg, u8 pri);
MCB_PTR     smxu_MsgGet(PCB_PTR pool, u8** bpp=NULL, u16 clrsz=0, MCB_PTR* mhp=NULL);
MCB_PTR     smxu_MsgMake(u8* bp, u32 bs=-1, u8 pri=0, MCB_PTR* mhp=NULL);
u32         smxu_MsgPeek(MCB_PTR msg, SMX_PK_PAR par);
MCB_PTR     smxu_MsgReceive(XCB_PTR xchg, u8** bpp=NULL, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
void        smxu_MsgReceiveStop(XCB_PTR xchg, u8** bpp=NULL, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
BOOLEAN     smxu_MsgRel(MCB_PTR msg, u16 clrsz=0);
u32         smxu_MsgRelAll(TCB_PTR task);
BOOLEAN     smxu_MsgSend(MCB_PTR msg, XCB_PTR xchg, u8 pri, void* reply=NULL);
u8*         smxu_MsgUnmake(MCB_PTR msg, u32* bsp=NULL);

BOOLEAN     smxu_MsgXchgClear(XCB_PTR xchg);
XCB_PTR     smxu_MsgXchgCreate(SMX_XCHG_MODE mode, const char* name=NULL, XCB_PTR* xhp=NULL);
BOOLEAN     smxu_MsgXchgDelete(XCB_PTR* xhp);
u32         smxu_MsgXchgPeek(XCB_PTR xchg, SMX_PK_PAR par);

BOOLEAN     smxu_MutexClear(MUCB_PTR mtx);
MUCB_PTR    smxu_MutexCreate(u8 pi, u8 ceiling, const char* name=NULL, MUCB_PTR* muhp=NULL);
BOOLEAN     smxu_MutexDelete(MUCB_PTR* muhp);
BOOLEAN     smxu_MutexFree(MUCB_PTR mtx);
BOOLEAN     smxu_MutexGet(MUCB_PTR mtx, u32 timeout=SMX_TMO_DFLT);
void        smxu_MutexGetStop(MUCB_PTR mtx, u32 timeout=SMX_TMO_DFLT);
BOOLEAN     smxu_MutexRel(MUCB_PTR mtx);

BOOLEAN     smxu_PipeClear(PICB_PTR pipe);
PICB_PTR    smxu_PipeCreate(void* ppb, u8 width, u16 length, const char* name=NULL, PICB_PTR* php=NULL);
void*       smxu_PipeDelete(PICB_PTR* php);
BOOLEAN     smxu_PipeGet8(PICB_PTR pipe, u8* bp);
u32         smxu_PipeGet8M(PICB_PTR pipe, u8* bp, u32 lim);
void        smxu_PipeGetPkt(PICB_PTR pipe, void* pdst);
BOOLEAN     smxu_PipeGetPktWait(PICB_PTR pipe, void* pdst, u32 timeout=SMX_TMO_DFLT);
void        smxu_PipeGetPktWaitStop(PICB_PTR pipe, void* pdst, u32 timeout=SMX_TMO_DFLT);
u32         smxu_PipePeek(PICB_PTR pipe, SMX_PK_PAR par);
BOOLEAN     smxu_PipePut8(PICB_PTR pipe, u8 b);
u32         smxu_PipePut8M(PICB_PTR pipe, u8* bp, u32 lim);
void        smxu_PipePutPkt(PICB_PTR pipe, void* psrc);
BOOLEAN     smxu_PipePutPktWait(PICB_PTR pipe, void* psrc, u32 timeout=SMX_TMO_DFLT, u32 mode=0);
void        smxu_PipePutPktWaitStop(PICB_PTR pipe, void* psrc, u32 timeout=SMX_TMO_DFLT, u32 mode=0);
BOOLEAN     smxu_PipeResume(PICB_PTR pipe);

u8*         smxu_PBlockGetHeap(u32 sz, u8 sn, u32 attr, const char* name=NULL, u32 hn=0);
u8*         smxu_PBlockGetPool(PCB_PTR pool, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     smxu_PBlockMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     smxu_PBlockRelHeap(u8* bp, u8 sn, u32 hn=0);
BOOLEAN     smxu_PBlockRelPool(u8* bp, u8 sn, PCB_PTR pool=0, u32 clrsz=0);
MCB_PTR     smxu_PMsgGetHeap(u32 sz, u8** bpp, u8 sn, u32 attr, u32 hn=0, MCB_PTR* mhp=NULL);
MCB_PTR     smxu_PMsgGetPool(PCB_PTR pool, u8** bpp, u8 sn, u32 attr, MCB_PTR* mhp=NULL);
MCB_PTR     smxu_PMsgMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL, MCB_PTR* mhp=NULL);
MCB_PTR     smxu_PMsgReceive(XCB_PTR xchg, u8** bpp, u8 dsn, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
void        smxu_PMsgReceiveStop(XCB_PTR xchg, u8** bpp, u8 dsn, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
BOOLEAN     smxu_PMsgRel(MCB_PTR* mhp, u16 clrsz=0);
BOOLEAN     smxu_PMsgReply(MCB_PTR pmsg);
BOOLEAN     smxu_PMsgSend(MCB_PTR pmsg, XCB_PTR xchg, u8 pri=0, void* reply=NULL);
BOOLEAN     smxu_PMsgSendB(MCB_PTR pmsg, XCB_PTR xchg, u8 pri=0, void* reply=NULL);

BOOLEAN     smxu_SemClear(SCB_PTR sem);
SCB_PTR     smxu_SemCreate(SMX_SEM_MODE mode, u8 lim, const char* name=NULL, SCB_PTR* shp=NULL);
BOOLEAN     smxu_SemDelete(SCB_PTR* shp);
u32         smxu_SemPeek(SCB_PTR sem, SMX_PK_PAR par);
BOOLEAN     smxu_SemSignal(SCB_PTR sem);
BOOLEAN     smxu_SemTest(SCB_PTR sem, u32 timeout=SMX_TMO_DFLT);
void        smxu_SemTestStop(SCB_PTR sem, u32 timeout=SMX_TMO_DFLT);

u32         smxu_SysEtimeGet(u32 flags=0);
u32         smxu_SysPeek(SMX_PK_PAR par);
void*       smxu_SysPseudoHandleCreate(void);
u32         smxu_SysStimeGet(void);
SMX_CBTYPE  smxu_SysWhatIs(void* h);

BOOLEAN     smxu_TaskBump(TCB_PTR task, u8 pri);
TCB_PTR     smxu_TaskCreate(FUN_PTR fun, u8 pri, u32 tlssz_ssz, u32 fl_hn, 
                        const char* name=NULL, u8* bp=NULL, TCB_PTR* thp=NULL);
TCB_PTR     smxu_TaskCurrent(void);
BOOLEAN     smxu_TaskDelete(TCB_PTR* thp);
void*       smxu_TaskLocate(const TCB_PTR task);
BOOLEAN     smxu_TaskLock(void);
BOOLEAN     smxu_TaskLockClear(void);
u32         smxu_TaskPeek(TCB_PTR task, SMX_PK_PAR par);
BOOLEAN     smxu_TaskResume(TCB_PTR task);
BOOLEAN     smxu_TaskSet(TCB_PTR task, SMX_ST_PAR par, u32 val, u32 val2=0);
BOOLEAN     smxu_TaskSleep(u32 time);
void        smxu_TaskSleepStop(u32 time);
BOOLEAN     smxu_TaskStart(TCB_PTR task, u32 par);
BOOLEAN     smxu_TaskStartNew(TCB_PTR task, u32 par, u8 pri, FUN_PTR fun);
BOOLEAN     smxu_TaskStop(TCB_PTR task, u32 timeout=SMX_TMO_INF);
BOOLEAN     smxu_TaskSuspend(TCB_PTR task, u32 timeout=SMX_TMO_INF);
BOOLEAN     smxu_TaskUnlock(void);
BOOLEAN     smxu_TaskUnlockQuick(void);
BOOLEAN     smxu_TaskYield(void);

BOOLEAN     smxu_TimerDup(TMRCB_PTR* tmrbp, TMRCB_PTR tmra, const char* name=NULL);
u32         smxu_TimerPeek(TMRCB_PTR tmr, SMX_PK_PAR par);
BOOLEAN     smxu_TimerReset(TMRCB_PTR tmr, u32* tlp=NULL);
BOOLEAN     smxu_TimerSetLSR(TMRCB_PTR tmr, LSR_PTR lsr, SMX_TMR_OPT opt, u32 par=0);
BOOLEAN     smxu_TimerSetPulse(TMRCB_PTR tmr, u32 period, u32 width);
BOOLEAN     smxu_TimerStart(TMRCB_PTR* tmhp, u32 delay, u32 period, LSR_PTR lsr, 
                                                         const char* name=NULL);
BOOLEAN     smxu_TimerStartAbs(TMRCB_PTR* tmhp, u32 time, u32 period, LSR_PTR lsr, 
                                                         const char* name=NULL);
BOOLEAN     smxu_TimerStop(TMRCB_PTR tmr, u32* tlp=NULL);

#if defined(SMXPP)
u8*         smxppu_ObjGet(void);
BOOLEAN     smxppu_ObjRel(u8* bp);
#endif

void        smxu_EVBLogUser4(void* h, u32 p1, u32 p2, u32 p3, u32 p4);
void        smxu_EVBLogUser5(void* h, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5);
void        smxu_EVBLogUser6(void* h, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6);
void        smxu_EVBLogUserPrint(u32 time, u32 index);

BOOLEAN     sbu_IRQClear(int irq_num);
BOOLEAN     sbu_IRQMask(int irq_num);
BOOLEAN     sbu_IRQUnmask(int irq_num);
u32         sbu_Peek(SB_PK_PAR par);
u32         sbu_PtimeGet(void);
#if SB_CON_OUT
void        sbu_MsgOutConst(u32 mtype, const char* mp);
void        sbu_MsgOutVar(u32 mtype, const char* mp);
#endif
void        sbu_TMInit(void);
void        sbu_TMStart(u32* pts);
void        sbu_TMEnd(u32 ts, u32* ptm);

#if SMX_CFG_MPU
BOOLEAN     mpu_MPACreate(TCB_PTR task, MPA* tmp=NULL, u32 tmsk=MP_TMSK_DFLT, 
                                                  u32 mpasz=MP_MPU_ACTVSZ);
BOOLEAN     mpu_MPACreateLSR(LCB_PTR lsr, MPA* tmp=NULL, u32 tmsk=MP_TMSK_DFLT, 
                                                  u32 mpasz=MP_MPU_ACTVSZ);
BOOLEAN     mpu_MPASlotMove(u8 dn, u8 sn);
#if SMX_CFG_PORTAL
BOOLEAN     mpu_FPortalClose(FPCS* pch, u8 xsn);
BOOLEAN     mpu_FPortalOpen(FPCS* pch, u8 csn, u32 msz, u32 nmsg, u8 pri, 
                                  u32 tmo=SMX_TMO_INF, const char* rxname=NULL);
MCB_PTR     mpu_FPortalReceive(FPCS* pch, u8** dpp);
BOOLEAN     mpu_FPortalSend(FPCS* pch, MCB* pmsg);
BOOLEAN     mpu_FTPortalSend(FPCS* pch, u8* bp, MCB* pmsg);
void        mpu_PortalEM(PS* ph, PERRNO errnum, PERRNO* ep);
void        mpu_PortalLog(u32 id, u32 p1=0, u32 p2=0, u32 p3=0, u32 p4=0);
void        mpu_PortalRet(u32 id, u32 rv);
#endif
#endif

#ifdef __cplusplus
}
#endif

#endif /* SMX_XAPIU1_H */


#if SMX_CFG_MPU_ENABLE || 0  //USER: Change 0 to 1 to enable SVC calls even if MPU and privilege disabled.
/*===========================================================================*
*                         smx to smxu Mapping Macros                         *
*===========================================================================*/
         
#define smx_BlockGet(pool, bpp, clrsz, bhp)     smxu_BlockGet(pool, bpp, clrsz, bhp)
#define smx_BlockMake(pool, bp, bhp)            smxu_BlockMake(pool, bp, bhp)
#define smx_BlockPeek(blk, par)                 smxu_BlockPeek(blk, par)
#define smx_BlockRel(blk, clrsz)                smxu_BlockRel(blk, clrsz)
#define smx_BlockRelAll(task)                   smxu_BlockRelAll(task)
#define smx_BlockUnmake(pool, blk)              smxu_BlockUnmake(pool, blk)

#define smx_BlockPoolCreate(pp, num, sz, name, php)  smxu_BlockPoolCreate(pp, num, sz, name, php)
#define smx_BlockPoolDelete(php)                smxu_BlockPoolDelete(php)
#define smx_BlockPoolPeek(pool, par)            smxu_BlockPoolPeek(pool, par)

#define smx_EventFlagsPulse(eg, pmsk)           smxu_EventFlagsPulse(eg, pmsk)
#define smx_EventFlagsSet(eg, smsk, pcmsk)      smxu_EventFlagsSet(eg, smsk, pcmsk)
#define smx_EventFlagsTest(eg, tmsk, pcmsk, tmo)  smxu_EventFlagsTest(eg, tmsk, pcmsk, tmo)
#define smx_EventFlagsTestStop(eg, tmsk, pcmsk, tmo) smxu_EventFlagsTestStop(eg, tmsk, pcmsk, tmo)

#define smx_EventGroupClear(eg, imsk)           smxu_EventGroupClear(eg, imsk)
#define smx_EventGroupCreate(imsk, name, eghp)  smxu_EventGroupCreate(imsk, name, eghp)
#define smx_EventGroupDelete(eghp)              smxu_EventGroupDelete(eghp)
#define smx_EventGroupPeek(eg, par)             smxu_EventGroupPeek(eg, par)
#define smx_EventGroupSet(eg, par, v1, v2)      _Pragma("error\"smx_EventGroupSet() not available in umode\"")

#define smx_EventQueueClear(eq)                 smxu_EventQueueClear(eq)
#define smx_EventQueueCount(eq, cnt, tmo)       smxu_EventQueueCount(eq, cnt, tmo)
#define smx_EventQueueCountStop(eq, cnt, tmo)   smxu_EventQueueCountStop(eq, cnt, tmo)
#define smx_EventQueueCreate(name, eqhp)        smxu_EventQueueCreate(name, eqhp)
#define smx_EventQueueDelete(eqhp)              smxu_EventQueueDelete(eqhp)
#define smx_EventQueueSignal(eq)                smxu_EventQueueSignal(eq)

#define smx_HeapBinPeek(binno, par, hn)         smxu_HeapBinPeek(binno, par, hn)
#define smx_HeapBinScan(binno, fnum, bnum, hn)  smxu_HeapBinScan(binno, fnum, bnum, hn)
#define smx_HeapBinSeed(num, bsz)               _Pragma("error\"smx_HeapBinSeed() not available in umode\"")
#define smx_HeapBinSort(binno, fnum, hn)        smxu_HeapBinSort(binno, fnum, hn)
#define smx_HeapCalloc(num, sz, an, hn)         smxu_HeapCalloc(num, sz, an, hn)
#define smx_HeapChunkPeek(vp, par, hn)          smxu_HeapChunkPeek(vp, par, hn)
#define smx_HeapExtend(xsz, xp)                 _Pragma("error\"smx_HeapExtend() not available in umode\"")
#define smx_HeapFree(bp, hn)                    smxu_HeapFree(bp, hn)
#define smx_HeapInit(sz, hp)                    _Pragma("error\"smx_HeapInit() not available in umode\"")
#define smx_HeapMalloc(sz, an, hn)              smxu_HeapMalloc(sz, an, hn)
#define smx_HeapPeek(par, hn)                   smxu_HeapPeek(par, hn)
#define smx_HeapRealloc(cbp, sz, an, hn)        smxu_HeapRealloc(cbp, sz, an, hn)
#define smx_HeapRecover(sz, fnum)               _Pragma("error\"smx_HeapRecover() not available in umode\"")
#define smx_HeapScan(cp, fnum, bnum)            _Pragma("error\"smx_HeapScan() not available in umode\"")
#define smx_HeapSet(par, val)                   _Pragma("error\"smx_HeapSet() not available in umode\"")

#define smx_HTAdd(h, name)                      smxu_HTAdd(h, name)
#define smx_HTDelete(h)                         smxu_HTDelete(h)
#define smx_HTGetHandle(name)                   smxu_HTGetHandle(name)
#define smx_HTGetName(h)                        smxu_HTGetName(h)
#define smx_HTInit()                            _Pragma("error\"smx_HTInit() not available in umode\"")

#define smx_LSRCreate(fun, htask, ssz, flags, name, lhp)  _Pragma("error\"smx_LSRCreate() not available in umode\"")
#define smx_LSRDelete(lhp)                      _Pragma("error\"smx_LSRDelete() not available in umode\"")
#define smx_LSRInvoke(lsr, par)                 _Pragma("error\"smx_LSRInvoke() not available in umode\"")
#define smx_LSRsOff(void)                       _Pragma("error\"smx_LSRsOff() not available in umode\"")
#define smx_LSRsOn(void)                        _Pragma("error\"smx_LSRsOn() not available in umode\"")

#define smx_MsgBump(msg, pri)                   smxu_MsgBump(msg, pri)
#define smx_MsgGet(pool, bpp, clrsz, mhp)       smxu_MsgGet(pool, bpp, clrsz, mhp)
#define smx_MsgMake(bp, bs, pri, mhp)           smxu_MsgMake(bp, bs, pri, mhp)
#define smx_MsgPeek(msg, par)                   smxu_MsgPeek(msg, par)
#define smx_MsgReceive(xchg, bpp, tmo, mhp)     smxu_MsgReceive(xchg, bpp, tmo, mhp)
#define smx_MsgReceiveStop(xchg, bpp, tmo, mhp) smxu_MsgReceiveStop(xchg, bpp, tmo, mhp)
#define smx_MsgRel(msg, clrsz)                  smxu_MsgRel(msg, clrsz)
#define smx_MsgRelAll(task)                     smxu_MsgRelAll(task)
#define smx_MsgSend(msg, xchg, pri, reply)      smxu_MsgSend(msg, xchg, pri, reply)
#define smx_MsgUnmake(msg, bsp)                 smxu_MsgUnmake(msg, bsp)

#define smx_MsgXchgClear(xchg)                  smxu_MsgXchgClear(xchg)
#define smx_MsgXchgCreate(mode, name, xhp)      smxu_MsgXchgCreate(mode, name, xhp)
#define smx_MsgXchgDelete(xhp)                  smxu_MsgXchgDelete(xhp)
#define smx_MsgXchgPeek(xchg, par)              smxu_MsgXchgPeek(xchg, par)
#define smx_MsgXchgSet(xchg, par, v1, v2)       _Pragma("error\"smx_MsgXchgSet() not available in umode\"")

#define smx_MutexClear(mtx)                     smxu_MutexClear(mtx)
#define smx_MutexCreate(pi, ceiling, name, muhp)  smxu_MutexCreate(pi, ceiling, name, muhp)
#define smx_MutexDelete(muhp)                   smxu_MutexDelete(muhp)
#define smx_MutexFree(mtx)                      smxu_MutexFree(mtx)
#define smx_MutexGet(mtx, tmo)                  smxu_MutexGet(mtx, tmo)
#define smx_MutexGetStop(mtx, tmo)              smxu_MutexGetStop(mtx, tmo)
#define smx_MutexRel(mtx)                       smxu_MutexRel(mtx)

#define smx_PipeClear(pipe)                     smxu_PipeClear(pipe)
#define smx_PipeCreate(ppb, w, l, name, php)    smxu_PipeCreate(ppb, w, l, name, php)
#define smx_PipeDelete(php)                     smxu_PipeDelete(php)
#define smx_PipeGet8(pipe, bp)                  smxu_PipeGet8(pipe, bp)
#define smx_PipeGet8M(pipe, bp, lim)            smxu_PipeGet8M(pipe, bp, lim)
#define smx_PipeGetPkt(pipe, pdst)              smxu_PipeGetPkt(pipe, pdst)
#define smx_PipeGetPktWait(pipe, pdst, tmo)     smxu_PipeGetPktWait(pipe, pdst, tmo)
#define smx_PipeGetPktWaitStop(pipe, pdst, tmo) smxu_PipeGetPktWaitStop(pipe, pdst, tmo)
#define smx_PipePeek(pipe, par)                 smxu_PipePeek(pipe, par)
#define smx_PipePut8(pipe, b)                   smxu_PipePut8(pipe, b)
#define smx_PipePut8M(pipe, bp, lim)            smxu_PipePut8M(pipe, bp, lim)
#define smx_PipePutPkt(pipe, psrc)              smxu_PipePutPkt(pipe, psrc)
#define smx_PipePutPktWait(pipe, psrc, tmo, mode)      smxu_PipePutPktWait(pipe, psrc, tmo, mode)
#define smx_PipePutPktWaitStop(pipe, psrc, tmo, mode)  smxu_PipePutPktWaitStop(pipe, psrc, tmo, mode)
#define smx_PipeResume(pipe)                    smxu_PipeResume(pipe)
#define smx_PipeSet(pipe, par, v1, v2)          _Pragma("error\"smx_PipeSet() not available in umode\"")

#define smx_PBlockGetHeap(sz, sn, attr, name, hn) smxu_PBlockGetHeap(sz, sn, attr, name, hn)
#define smx_PBlockGetPool(pool, sn, attr, name) smxu_PBlockGetPool(pool, sn, attr, name)
#define smx_PBlockMake(bp, sz, sn, attr, name)  smxu_PBlockMake(bp, sz, sn, attr, name)
#define smx_PBlockRelHeap(bp, sn, hn)           smxu_PBlockRelHeap(bp, sn, hn)
#define smx_PBlockRelPool(bp, sn, pool, clrsz)  smxu_PBlockRelPool(bp, sn, pool, clrsz)
#define smx_PMsgGetHeap(sz, bpp, sn, attr, hn, mhp)  smxu_PMsgGetHeap(sz, bpp, sn, attr, hn, mhp)
#define smx_PMsgGetPool(pool, bpp, sn, attr, mhp)    smxu_PMsgGetPool(pool, bpp, sn, attr, mhp)
#define smx_PMsgMake(bp, sz, sn, attr, name, mhp)    smxu_PMsgMake(bp, sz, sn, attr, name, mhp)
#define smx_PMsgReceive(xchg, bpp, dsn, tmo, mhp)    smxu_PMsgReceive(xchg, bpp, dsn, tmo, mhp)
#define smx_PMsgReceiveStop(xchg, bpp, dsn, tmo, mhp) smxu_PMsgReceiveStop(xchg, bpp, dsn, tmo, mhp)
#define smx_PMsgRel(mhp, clrsz)                 smxu_PMsgRel(mhp, clrsz)
#define smx_PMsgReply(msg)                      smxu_PMsgReply(msg)
#define smx_PMsgSend(msg, xchg, pri, reply)     smxu_PMsgSend(msg, xchg, pri, reply)
#define smx_PMsgSendB(msg, xchg, pri, reply)    smxu_PMsgSendB(msg, xchg, pri, reply)

#define smx_SemClear(sem)                       smxu_SemClear(sem)
#define smx_SemCreate(mode, lim, name, shp)     smxu_SemCreate(mode, lim, name, shp)
#define smx_SemDelete(shp)                      smxu_SemDelete(shp)
#define smx_SemPeek(sem, par)                   smxu_SemPeek(sem, par)
#define smx_SemSet(sem, par, v1, v2)            _Pragma("error\"smx_SemSet() not available in umode\"")
#define smx_SemSignal(sem)                      smxu_SemSignal(sem)
#define smx_SemTest(sem, tmo)                   smxu_SemTest(sem, tmo)
#define smx_SemTestStop(sem, tmo)               smxu_SemTestStop(sem, tmo)

#define smx_SysEtimeGet(flags)                  smxu_SysEtimeGet(flags)
#define smx_SysPeek(par)                        smxu_SysPeek(par)
#define smx_SysPowerDown(power_mode)            _Pragma("error\"smx_SysPowerDown() not available in umode\"")
#define smx_SysPseudoHandleCreate()             smxu_SysPseudoHandleCreate()
#define smx_SysStimeGet()                       smxu_SysStimeGet()
#define smx_SysWhatIs(h)                        smxu_SysWhatIs(h)

#define smx_TaskBump(task, pri)                 smxu_TaskBump(task, pri)
#define smx_TaskCreate(fun, pri, tlssz_ssz, fl_hn, name, bp, thp) \
                                                smxu_TaskCreate(fun, pri, tlssz_ssz, fl_hn, name, bp, thp)
#define smx_TaskCurrent()                       smxu_TaskCurrent()
#define smx_TaskDelete(thp)                     smxu_TaskDelete(thp)
#define smx_TaskLocate(task)                    smxu_TaskLocate(task)
#define smx_TaskLock()                          _Pragma("error\"smx_TaskLock() not available in umode\"")
#define smx_TaskLockClear()                     _Pragma("error\"smx_TaskLockClear() not available in umode\"")
#define smx_TaskPeek(task, par)                 smxu_TaskPeek(task, par)
#define smx_TaskResume(task)                    smxu_TaskResume(task)
#define smx_TaskSet(task, par, val, val2)       _Pragma("error\"smx_TaskSet() not available in umode\"")
#define smx_TaskSleep(time)                     smxu_TaskSleep(time)
#define smx_TaskSleepStop(time)                 smxu_TaskSleepStop(time)
#define smx_TaskStart(task, par)                smxu_TaskStart(task, par)
#define smx_TaskStartNew(task, par, pri,  fun)  smxu_TaskStartNew(task, par, pri,  fun)
#define smx_TaskStop(task, tmo)                 smxu_TaskStop(task, tmo)
#define smx_TaskSuspend(task, tmo)              smxu_TaskSuspend(task, tmo)
#define smx_TaskUnlock()                        _Pragma("error\"smx_TaskUnlock() not available in umode\"")
#define smx_TaskUnlockQuick()                   _Pragma("error\"smx_TaskUnlockQuick() not available in umode\"")
#define smx_TaskYield()                         smxu_TaskYield()

#define smx_TimerDup(tmrbp, tmra, name)         smxu_TimerDup(tmrbp, tmra, name)
#define smx_TimerPeek(tmr, par)                 smxu_TimerPeek(tmr, par)
#define smx_TimerReset(tmr, tlp)                smxu_TimerReset(tmr, tlp)
#define smx_TimerSetLSR(tmr, lsr, opt, par)     _Pragma("error\"smx_TimerSetLSR() not available in umode\"")
#define smx_TimerSetPulse(tmr, per, width)      smxu_TimerSetPulse(tmr, per, width)
#define smx_TimerStart(tmhp, dly, per, lsr, name)     smxu_TimerStart(tmhp, dly, per, lsr, name)
#define smx_TimerStartAbs(tmhp, time, per, lsr, name) smxu_TimerStartAbs(tmhp, time, per, lsr, name)
#define smx_TimerStop(tmr, tlp)                 smxu_TimerStop(tmr, tlp)

#if defined(SMXPP)
#define smxpp_ObjGet()                          smxppu_ObjGet()
#define smxpp_ObjRel(bp)                        smxppu_ObjRel(bp)
#endif

#define mp_MPACreate(task, tmp, tmsk, mpasz)    mpu_MPACreate(task, tmp, tmsk, mpasz)
#define mp_MPACreateLSR(lsr, tmp, tmsk, mpasz)  mpu_MPACreateLSR(lsr, tmp, tmsk, mpasz)
#define mp_MPASlotMove(dn, sn)                  mpu_MPASlotMove(dn, sn)

#define mp_FPortalClose(pch, xsn)               mpu_FPortalClose(pch, xsn)
#define mp_FPortalOpen(pch, csn, msz, nmsg, pri, tmo, rxname)  mpu_FPortalOpen(pch, csn, msz, nmsg, pri, tmo, rxname)
#define mp_FPortalReceive(pch, dpp)             mpu_FPortalReceive(pch, dpp)
#define mp_FPortalSend(pch, pmsg)               mpu_FPortalSend(pch, pmsg)
#define mp_FTPortalSend(pch, bp, pmsg)          mpu_FTPortalSend(pch, bp, pmsg)
#define mp_PortalEM(ph, errnum, ep)             mpu_PortalEM(ph, errnum, ep)
#define mp_PortalLog(id, p1, p2, p3, p4)        mpu_PortalLog(id, p1, p2, p3, p4)
#define mp_PortalRet(id, rv)                    mpu_PortalRet(id, rv)

/* only implemented User4 to save IDs, since only 256 for SVC, and > 5 pars not supported yet */
#define smx_EVBLogUser0(h)                      smxu_EVBLogUser4(h, 0, 0, 0, 0)
#define smx_EVBLogUser1(h, p1)                  smxu_EVBLogUser4(h, p1, 0, 0, 0)
#define smx_EVBLogUser2(h, p1, p2)              smxu_EVBLogUser4(h, p1, p2, 0, 0)
#define smx_EVBLogUser3(h, p1, p2, p3)          smxu_EVBLogUser4(h, p1, p2, p3, 0)
#define smx_EVBLogUser4(h, p1, p2, p3, p4)      smxu_EVBLogUser4(h, p1, p2, p3, p4)
#define smx_EVBLogUser5(h, p1, p2, p3, p4, p5)     smxu_EVBLogUser5(h, p1, p2, p3, p4, p5)
#define smx_EVBLogUser6(h, p1, p2, p3, p4, p5, p6) smxu_EVBLogUser6(h, p1, p2, p3, p4, p5, p6)
#define smx_EVBLogUserPrint(time, index)        smxu_EVBLogUserPrint(time, index)

#define sb_IRQClear(irq)                        sbu_IRQClear(irq)
#define sb_IRQMask(irq)                         sbu_IRQMask(irq)
#define sb_IRQUnmask(irq)                       sbu_IRQUnmask(irq)
#define sb_Peek(par)                            sbu_Peek(par)
#define sb_PtimeGet()                           sbu_PtimeGet()
#if SB_CON_OUT
#define sb_MsgOutConst(mtype, mp)               sbu_MsgOutConst(mtype, mp)
#define sb_MsgOutVar(mtype, mp)                 sbu_MsgOutVar(mtype, mp)
#endif
#define sb_TMInit()                             sbu_TMInit()         /* in bbase.c; not SVC call */
#define sb_TMStart(pts)                         sbu_TMStart(pts)     /* in bbase.c; not SVC call */
#define sb_TMEnd(ts, ptm)                       sbu_TMEnd(ts, ptm)   /* in bbase.c; not SVC call */
#endif /* SMX_CFG_MPU_ENABLE */

#undef SMX_XAPIP_H /* allow xapip.h to be included again */

#endif /* SMX_CFG_MPU */
#endif /* SMX_XAPIU_H */

