/*
* acfg.h                                                 Version 5.2.0
*
* Application Configuration Constants (Medium Values)
*
* See the SMX Quick Start for documentation of these settings.
*
* The following configuration constants work with pre-compiled smx libraries
* via the smx_cf structure, which is defined in main.c. See xsmx\xcfg.h for
* configuration constants that control the compilation of smx libraries.
*
* Copyright (c) 1989-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Ralph Moore and David Moore
*
*****************************************************************************/

#ifndef SMX_ACFG_H
#define SMX_ACFG_H

/* Sizes and Quantities */

#if defined(__IAR_SYSTEMS_ICC__)
#pragma section = "heap0"
#endif
#if SMX_CFG_MPU
#define SMX_CFG_HEAP_ADDRESS    ((u32)__section_begin("heap0")) /* heap starting address */
#else
#define SMX_CFG_HEAP_ADDRESS    ((u32)&heap0) /* heap starting address */
#endif

#define SMX_CFG_HEAP_SPACE     (7000 + SMX_CFG_HT_SIZE*8 + SMX_CFG_NUM_STACKS*SMX_CFG_STACK_BLK_SIZE)
#define SMX_CFG_HEAP_DC_SIZE       (SMX_CFG_HEAP_SPACE/32 & ~0x7) /* initial donor chunk size <1> */
#define SMX_CFG_HEAP_CSZ_MAX       4136                           /* maximum dynamic chunk size <3> */
#define SMX_CFG_HEAP_USE_MAX       (SMX_CFG_HEAP_SPACE*3/4)       /* level to turn on cmerge */
#define SMX_CFG_HEAP_USE_MIN       (SMX_CFG_HEAP_USE_MAX - 256)   /* level to turn off cmerge */
#define SMX_CFG_HT_SIZE             20   /* number of handles in the handle table */
#define SMX_CFG_LQ_SIZE             10
#define SMX_CFG_NUM_BLOCKS          10   /* number of blocks of all sizes */
#define SMX_CFG_NUM_EQS              2
#define SMX_CFG_NUM_EVGS             2
#define SMX_CFG_NUM_LSRS            10
#define SMX_CFG_NUM_MSGS            10
#define SMX_CFG_NUM_MTXS             6
#define SMX_CFG_NUM_SEMS             4
#define SMX_CFG_NUM_PIPES            2
#define SMX_CFG_NUM_POOLS            3   /* number of block and msg pools */
#define SMX_CFG_NUM_STACKS           2   /* number of stacks in stack pool */
#define SMX_CFG_NUM_TASKS            8   /* includes Nulltask and Idle */
#define SMX_CFG_NUM_TIMERS           5   /* number of timer control blocks, TMCBs */
#define SMX_CFG_NUM_XCHGS            4
#define SMX_CFG_PP_OBJ_NUM           5   /* number of smx++ objects that can be allocated from GlobalPool */
#define SMX_CFG_PP_OBJ_SIZE         16   /* typical size of smx++ objects to be allocated from GlobalPool */
#define SMX_CFG_SA_PRINT_RING_SIZE 1000  /* size of the smxAware print ring (bytes) */
#define SMX_CFG_STACK_PAD_SIZE       20  /* size of stack pads for all stacks <2> */
#define SMX_CFG_STACK_SIZE          400  /* size of stacks in the stack pool <2> */
#define SMX_CFG_STACK_SIZE_IDLE  (SMX_CFG_STACK_SIZE + 400) /* size of idle task stack (for init/exit too) <2> */

#if SMX_CFG_MPU
#define SMX_CFG_STACK_BLK_SIZE 512
#if SMX_CFG_STACK_BLK_SIZE < (SMX_CFG_STACK_SIZE + SMX_CFG_STACK_PAD_SIZE + SMX_RSA_SIZE)
#error Choose next larger power of 2 for STACK_BLK_SIZE.
#endif
#else
#define SMX_CFG_STACK_BLK_SIZE ((SMX_CFG_STACK_SIZE + SMX_CFG_STACK_PAD_SIZE + SMX_RSA_SIZE + 7) & 0xFFFFFFF8)
#endif

#define SMX_CFG_TICKS_PER_SEC       100 

/* Safety Checks */
#if (SMX_CFG_STACK_PAD_SIZE % 4 != 0) || (SMX_CFG_STACK_SIZE % 4 != 0) || (SMX_CFG_STACK_SIZE_IDLE % 4 != 0)
#error SMX_CFG_STACK SIZE settings must be multiples of 4.
#endif


/* Notes:
   1. Must be a multiple of 8 and at least 24.
   2. Must be a multiple of 8.
   3. Replace default value with actual maximum dynamic chunk size for your application.
      Dynamic means a chunk which is allocated and freed as the application runs;
      it does not include chunks permanently allocated at the start.
*/

#endif /* SMX_ACFG_H */
