/*
* xapi.h                                                    Version 5.2.0
*
* smx API functions and macros for pmode (privilege mode) only,
* and common functions and macros for pcode and ucode. 
*
* Copyright (c) 1989-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Ralph Moore, David Moore
*
*****************************************************************************/

#ifndef SMX_XAPI_H
#define SMX_XAPI_H

#ifdef __cplusplus
extern "C" {
#endif

/*===========================================================================*
*                         smx API SSRs and Functions                         *
*===========================================================================*/

BCB_PTR     smx_BlockGet(PCB_PTR pool, u8** bpp, u32 clrsz=0, BCB_PTR* bhp=NULL);
BCB_PTR     smx_BlockMake(PCB_PTR pool, u8* bp, BCB_PTR* bhp=NULL);
u32         smx_BlockPeek(BCB_PTR blk, SMX_PK_PAR par);
BOOLEAN     smx_BlockRel(BCB_PTR blk, u16 clrsz=0);
u32         smx_BlockRelAll(TCB_PTR task);
u8*         smx_BlockUnmake(PCB_PTR* pool, BCB_PTR blk);

PCB_PTR     smx_BlockPoolCreate(u8* p, u16 num, u16 size, const char* name=NULL, PCB_PTR* php=NULL);
u8*         smx_BlockPoolDelete(PCB_PTR* php);
u32         smx_BlockPoolPeek(PCB_PTR pool, SMX_PK_PAR par);

BOOLEAN     smx_CBPoolsCreate(void);

BOOLEAN     smx_EventFlagsPulse(EGCB_PTR eg, u16 pulse_mask);
BOOLEAN     smx_EventFlagsSet(EGCB_PTR eg, u16 set_mask, u16 pre_clear_mask);
u32         smx_EventFlagsTest(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout=SMX_TMO_DFLT);
void        smx_EventFlagsTestStop(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout=SMX_TMO_DFLT);

BOOLEAN     smx_EventGroupClear(EGCB_PTR eg, u16 init_mask);
EGCB_PTR    smx_EventGroupCreate(u32 init_mask, const char* name=NULL, EGCB_PTR* eghp=NULL);
BOOLEAN     smx_EventGroupDelete(EGCB_PTR* eghp);
u32         smx_EventGroupPeek(EGCB_PTR eg, SMX_PK_PAR par);
BOOLEAN     smx_EventGroupSet(EGCB_PTR eg, SMX_ST_PAR par, u32 v1, u32 v2=0);

BOOLEAN     smx_EventQueueClear(EQCB_PTR eq);
BOOLEAN     smx_EventQueueCount(EQCB_PTR eq, u32 count, u32 timeout=SMX_TMO_DFLT);
void        smx_EventQueueCountStop(EQCB_PTR eq, u32 count, u32 timeout=SMX_TMO_DFLT);
EQCB_PTR    smx_EventQueueCreate(const char* name=NULL, EQCB_PTR* eqhp=NULL);
BOOLEAN     smx_EventQueueDelete(EQCB_PTR* eqhp);
BOOLEAN     smx_EventQueueSignal(EQCB_PTR eq);

void        smx_Go(void);

u32         smx_HeapBinPeek(u32 binno, EH_PK_PAR par, u32 hn=0);
BOOLEAN     smx_HeapBinScan(u32 binno, u32 fnum, u32 bnum, u32 hn=0);
BOOLEAN     smx_HeapBinSeed(u32 num, u32 bsz, u32 hn=0);
BOOLEAN     smx_HeapBinSort(u32 binno, u32 fnum, u32 hn=0);
void*       smx_HeapCalloc(u32 num, u32 sz, u32 an=0, u32 hn=0);
u32         smx_HeapChunkPeek(void* vp, EH_PK_PAR par, u32 hn=0);
BOOLEAN     smx_HeapExtend(u32 xsz, u8* xp, u32 hn=0);
BOOLEAN     smx_HeapFree(void* bp, u32 hn=0);
u32         smx_HeapInit(u32 sz, u32 dcsz, u8* hp, EHV_PTR vp, u32 mode);
void*       smx_HeapMalloc(u32 sz, u32 an=0, u32 hn=0);
u32         smx_HeapPeek(EH_PK_PAR par, u32 hn=0);
void*       smx_HeapRealloc(void* cbp, u32 sz, u32 an=0, u32 hn=0);
BOOLEAN     smx_HeapRecover(u32 sz, u32 num, u32 an=0, u32 hn=0);
BOOLEAN     smx_HeapScan(CCB_PTR cp, u32 fnum, u32 bnum, u32 hn=0);
BOOLEAN     smx_HeapSet(EH_ST_PAR par, u32 val, u32 hn=0);

BOOLEAN     smx_HTAdd(void* h, const char* name);
BOOLEAN     smx_HTDelete(void* h);
void*       smx_HTGetHandle(const char* name);
const char* smx_HTGetName(void* h);
void        smx_HTInit(void);

LCB_PTR     smx_LSRCreate(FUN_PTR fun, TCB_PTR htask, u32 ssz, u32 flags, 
                                                const char* name, LCB_PTR* lhp=NULL);
BOOLEAN     smx_LSRDelete(LCB_PTR* lhp);
BOOLEAN     smx_LSRInvoke(LCB_PTR lsr, u32 par=0);
void        smx_LSRInvokeF(LCB_PTR lsr, u32 par=0);
void        smx_LSRsOff(void);
BOOLEAN     smx_LSRsOn(void);

BOOLEAN     smx_MsgBump(MCB_PTR msg, u8 pri);
MCB_PTR     smx_MsgGet(PCB_PTR pool, u8** bpp=NULL, u16 clrsz=0, MCB_PTR* mhp=NULL);
MCB_PTR     smx_MsgMake(u8* bp, u32 bs=-1, u8 pri=0, MCB_PTR* mhp=NULL);
u32         smx_MsgPeek(MCB_PTR msg, SMX_PK_PAR par);
MCB_PTR     smx_MsgReceive(XCB_PTR xchg, u8** bpp=NULL, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
void        smx_MsgReceiveStop(XCB_PTR xchg, u8** bpp=NULL, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
BOOLEAN     smx_MsgRel(MCB_PTR msg, u16 clrsz=0);
u32         smx_MsgRelAll(TCB_PTR task);
BOOLEAN     smx_MsgSend(MCB_PTR msg, XCB_PTR xchg, u8 pri=0, void* reply=NULL);
u8*         smx_MsgUnmake(MCB_PTR msg, u32* bsp=NULL);

BOOLEAN     smx_MsgXchgClear(XCB_PTR xchg);
XCB_PTR     smx_MsgXchgCreate(SMX_XCHG_MODE mode, const char* name=NULL, XCB_PTR* xhp=NULL);
BOOLEAN     smx_MsgXchgDelete(XCB_PTR* xhp);
u32         smx_MsgXchgPeek(XCB_PTR xchg, SMX_PK_PAR par);
BOOLEAN     smx_MsgXchgSet(XCB_PTR xchg, SMX_ST_PAR par, u32 v1, u32 v2=0);

BOOLEAN     smx_MutexClear(MUCB_PTR mtx);
MUCB_PTR    smx_MutexCreate(u8 pi, u8 ceiling, const char* name=NULL, MUCB_PTR* muhp=NULL);
BOOLEAN     smx_MutexDelete(MUCB_PTR* muhp);
BOOLEAN     smx_MutexFree(MUCB_PTR mtx);
BOOLEAN     smx_MutexGet(MUCB_PTR mtx, u32 timeout=SMX_TMO_DFLT);
BOOLEAN     smx_MutexGetFast(MUCB_PTR mtx, u32 timeout=SMX_TMO_DFLT);
void        smx_MutexGetStop(MUCB_PTR mtx, u32 timeout=SMX_TMO_DFLT);
BOOLEAN     smx_MutexRel(MUCB_PTR mtx);
void        smx_MutexRelFast(MUCB_PTR mtx);

BOOLEAN     smx_PipeClear(PICB_PTR pipe);
PICB_PTR    smx_PipeCreate(void* ppb, u8 width, u16 length, const char* name=NULL, PICB_PTR* php=NULL);
void*       smx_PipeDelete(PICB_PTR* php);
BOOLEAN     smx_PipeGet8(PICB_PTR pipe, u8* bp);
u32         smx_PipeGet8M(PICB_PTR pipe, u8* bp, u32 lim);
BOOLEAN     smx_PipeGetPkt(PICB_PTR pipe, void* pdst);
BOOLEAN     smx_PipeGetPktWait(PICB_PTR pipe, void* pdst, u32 timeout=SMX_TMO_DFLT);
void        smx_PipeGetPktWaitStop(PICB_PTR pipe, void* pdst, u32 timeout=SMX_TMO_DFLT);
u32         smx_PipePeek(PICB_PTR pipe, SMX_PK_PAR par);
BOOLEAN     smx_PipePut8(PICB_PTR pipe, u8 b);
u32         smx_PipePut8M(PICB_PTR pipe, u8* bp, u32 lim);
BOOLEAN     smx_PipePutPkt(PICB_PTR pipe, void* psrc);
BOOLEAN     smx_PipePutPktWait(PICB_PTR pipe, void* psrc, u32 timeout=SMX_TMO_DFLT, u32 mode=0);
void        smx_PipePutPktWaitStop(PICB_PTR pipe, void* psrc, u32 timeout=SMX_TMO_DFLT, u32 mode=0);
BOOLEAN     smx_PipeResume(PICB_PTR pipe);
BOOLEAN     smx_PipeSet(PICB_PTR pipe, SMX_ST_PAR par, u32 v1, u32 v2=0);

#if SMX_CFG_MPU
u8*         smx_PBlockGetHeap(u32 sz, u8 sn, u32 attr, const char* name=NULL, u32 hn=0);
u8*         smx_PBlockGetPool(PCB_PTR pool, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     smx_PBlockMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     smx_PBlockRelHeap(u8* bp, u8 sn, u32 hn=0);
BOOLEAN     smx_PBlockRelPool(u8* bp, u8 sn, PCB_PTR pool=0, u32 clrsz=0);
MCB_PTR     smx_PMsgGetHeap(u32 sz, u8** bpp, u8 sn, u32 attr, u32 hn=0, MCB_PTR* mhp=NULL);
MCB_PTR     smx_PMsgGetPool(PCB_PTR pool, u8** bpp, u8 sn, u32 attr, MCB_PTR* mhp=NULL);
MCB_PTR     smx_PMsgMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL, MCB_PTR* mhp=NULL);
MCB_PTR     smx_PMsgReceive(XCB_PTR xchg, u8** bpp, u8 dsn, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
void        smx_PMsgReceiveStop(XCB_PTR xchg, u8** bpp, u8 dsn, u32 timeout=SMX_TMO_DFLT, MCB_PTR* mhp=NULL);
BOOLEAN     smx_PMsgRel(MCB_PTR* mhp, u16 clrsz=0);
BOOLEAN     smx_PMsgReply(MCB_PTR pmsg);
BOOLEAN     smx_PMsgSend(MCB_PTR pmsg, XCB_PTR xchg, u8 pri=0, void* reply=NULL);
BOOLEAN     smx_PMsgSendB(MCB_PTR pmsg, XCB_PTR xchg, u8 pri=0, void* reply=NULL);
#endif

BOOLEAN     smx_SemClear(SCB_PTR sem);
SCB_PTR     smx_SemCreate(SMX_SEM_MODE mode, u8 lim, const char* name=NULL, SCB_PTR* shp=NULL);
BOOLEAN     smx_SemDelete(SCB_PTR* shp);
u32         smx_SemPeek(SCB_PTR sem, SMX_PK_PAR par);
BOOLEAN     smx_SemSet(SCB_PTR sem, SMX_ST_PAR par, u32 v1, u32 v2=0);
BOOLEAN     smx_SemSignal(SCB_PTR sem);
BOOLEAN     smx_SemTest(SCB_PTR sem, u32 timeout=SMX_TMO_DFLT);
void        smx_SemTestStop(SCB_PTR sem, u32 timeout=SMX_TMO_DFLT);

u32         smx_SysEtimeGet(u32 flags=0);
u32         smx_SysPeek(SMX_PK_PAR par);
BOOLEAN     smx_SysPowerDown(u32 power_mode);
void*       smx_SysPseudoHandleCreate(void);
u32         smx_SysStimeGet(void);
SMX_CBTYPE  smx_SysWhatIs(void* h);

BOOLEAN     smx_TaskBump(TCB_PTR task, u8 pri);
TCB_PTR     smx_TaskCreate(FUN_PTR fun, u8 pri, u32 tlssz_ssz, u32 fl_hn, 
                     const char* name=NULL, u8* bp=NULL, TCB_PTR* thp=NULL);
TCB_PTR     smx_TaskCurrent(void);
BOOLEAN     smx_TaskDelete(TCB_PTR* thp);
void*       smx_TaskLocate(const TCB_PTR task);
BOOLEAN     smx_TaskLock(void);
BOOLEAN     smx_TaskLockClear(void);
u32         smx_TaskPeek(TCB_PTR task, SMX_PK_PAR par);
BOOLEAN     smx_TaskResume(TCB_PTR task);
BOOLEAN     smx_TaskSet(TCB_PTR task, SMX_ST_PAR par, u32 v1, u32 v2=0);
BOOLEAN     smx_TaskSleep(u32 time);
void        smx_TaskSleepStop(u32 time);
BOOLEAN     smx_TaskStart(TCB_PTR task, u32 par=0);
BOOLEAN     smx_TaskStartNew(TCB_PTR task, u32 par, u8 pri, FUN_PTR fun);
BOOLEAN     smx_TaskStop(TCB_PTR task, u32 timeout=SMX_TMO_INF);
BOOLEAN     smx_TaskSuspend(TCB_PTR task, u32 timeout=SMX_TMO_INF);
BOOLEAN     smx_TaskUnlock(void);
BOOLEAN     smx_TaskUnlockQuick(void);
#if SMX_CFG_MPU
BOOLEAN     smx_TaskYield(void);
#endif

BOOLEAN     smx_TimerDup(TMRCB_PTR* tmrbp, TMRCB_PTR tmra, const char* name=NULL);
u32         smx_TimerPeek(TMRCB_PTR tmr, SMX_PK_PAR par);
BOOLEAN     smx_TimerReset(TMRCB_PTR tmr, u32* tlp=NULL);
BOOLEAN     smx_TimerSetLSR(TMRCB_PTR tmr, LCB_PTR lsr, SMX_TMR_OPT opt, u32 par=0);
BOOLEAN     smx_TimerSetPulse(TMRCB_PTR tmr, u32 period, u32 width);
BOOLEAN     smx_TimerStart(TMRCB_PTR* tmhp, u32 delay, u32 period, LCB_PTR lsr, 
                                                         const char* name=NULL);
BOOLEAN     smx_TimerStartAbs(TMRCB_PTR* tmhp, u32 time, u32 period, LCB_PTR lsr, 
                                                         const char* name=NULL);
BOOLEAN     smx_TimerStop(TMRCB_PTR tmr, u32* tlp=NULL);


/*===========================================================================*
*                           smx Misc Functions                               *
*===========================================================================*/

void        smx_NullF(void);              /* NOP function */
BOOLEAN     smx_NullSSR(void);            /* NOP SSR */

#if SMX_CFG_PROFILE
void        smx_RTC_ISRStart(void);
void        smx_RTC_ISREnd(void);
void        smx_RTC_LSRStart(void);
void        smx_RTC_LSREnd(void);
void        smx_RTCTaskStart(void);
void        smx_RTCTaskStartID(void);
void        smx_RTCTaskEnd(void);
#endif


/*===========================================================================*
*                         smx Internal Functions                             *
*===========================================================================*/

void        smx_EM(SMX_ERRNO errnum, void* handle);

#if defined(SB_CPU_ARMM)
void        smx_EM_SVC(SMX_ERRNO errnum, void* handle);
#else
BOOLEAN     smx_EM_SS(SMX_ERRNO errnum, void* handle);
#endif

BOOLEAN     smx_HeapsInit(void);          /* app callback to init heaps */
BOOLEAN     smx_InSS(void);
void        smx_KeepTimeLSRMain(u32 par=0);
void        smx_TaskDeleteLSRMain(u32 taskp);
void        smx_TimeoutLSRMain(u32 par);

#if defined(SB_CPU_ARMM)
void        smx_MMF_Handler(void);        /* memory manage fault handler */
void        smx_UF_Handler(void);         /* usage fault handler (ARMM8) */
BOOLEAN     smx_TS_Ovfl(void);            /* task stack overflow (ARMM8) */
#endif

void        smx_PreSched(void);

#if SMX_CFG_PROFILE
void        smx_ProfileDisplay(void);
void        smx_ProfileInit(void);
void        smx_ProfileLSRMain(u32 par);
#endif

BOOLEAN     smx_SchedRunLSRs(u32 reload=0);  /* LSR scheduler */
void        smx_SchedRunTasks(void);         /* Task scheduler */
u32         smx_SSRExit(u32 ret, u32 id);    /* SSR exit */
u32         smx_SSRExitInt(u32 ret);         /* SSR exit internal */
void        smx_StackScan(void);             /* Scans a stack to set HWM. */
u32         smx_SVC(u32 ssr_id);             /* Invoke SWI SSR */

#if SMX_CFG_MPU
BOOLEAN     smx_TaskOpPermit(TCB_PTR task);
#endif

void        smx_TimeoutSet(TCB_PTR task, u32 timeout);

#if SMX_CFG_MPU
/* needed for macros below since xapiu.h hasn't been included yet */
u32         smxu_SysPeek(SMX_PK_PAR par);
#endif

#if SMX_CFG_TOKENS
BOOLEAN     smx_TokenTest(TCB_PTR task, u32 hp, BOOLEAN priv);  /* Verify task has token */
#endif

#ifdef __cplusplus
}
#endif

/* Overloaded SSREnter() functions */
void        smx_SSREnter(u32 id);
void        smx_SSREnter(u32 id, u32 p1);
void        smx_SSREnter(u32 id, u32 p1, u32 p2);
void        smx_SSREnter(u32 id, u32 p1, u32 p2, u32 p3);
void        smx_SSREnter(u32 id, u32 p1, u32 p2, u32 p3, u32 p4);
void        smx_SSREnter(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5);
void        smx_SSREnter(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6);
void        smx_SSREnter(u32 id, u32 p1, u32 p2, u32 p3, u32 p4, u32 p5, u32 p6, u32 p7);


/*===========================================================================*
*                        Macros for Application Use                          *
*===========================================================================*/

/* Conversion Macros <1> */
#define     smx_ConvMsecToTicks(ms)         (((ms)*SMX_CFG_TICKS_PER_SEC + 999) / 1000) /*<4>*/
#define     smx_ConvMsecToTicksRound(ms)    (((ms)*SMX_CFG_TICKS_PER_SEC + 500) / 1000) /*<5>*/
#define     smx_ConvTicksToMsec(t)          ((1000*(t) + SMX_CFG_TICKS_PER_SEC-1) / SMX_CFG_TICKS_PER_SEC) /*<4>*/
#define     smx_ConvTicksToMsecRound(t)     ((1000*(t) + SMX_CFG_TICKS_PER_SEC/2) / SMX_CFG_TICKS_PER_SEC) /*<5>*/

/* Profiling Macros */
#if SMX_CFG_PROFILE || SMX_CFG_RTLIM
#define     smx_RTC_TASK_START()    smx_RTCTaskStart();
#define     smx_RTC_TASK_START_ID() smx_RTCTaskStartID();
#define     smx_RTC_TASK_END()      smx_RTCTaskEnd();
#else
#define     smx_RTC_TASK_START()
#define     smx_RTC_TASK_START_ID()
#define     smx_RTC_TASK_END()
#endif
#if SMX_CFG_PROFILE
#define     smx_RTC_ISR_START()     smx_RTC_ISRStart();
#define     smx_RTC_ISR_END()       smx_RTC_ISREnd();
#define     smx_RTC_LSR_START()     smx_RTC_LSRStart();
#define     smx_RTC_LSR_END()       smx_RTC_LSREnd();
#else
#define     smx_RTC_ISR_START()
#define     smx_RTC_ISR_END()
#define     smx_RTC_LSR_START()
#define     smx_RTC_LSR_END()
#endif


/*===========================================================================*
*                             Macros for SSRs <2>                            *
*===========================================================================*/

#if defined(SB_CPU_ARMM)
#define     smx_ERROR(errnum, h)    smx_EM_SVC(errnum, h); /* via smx_SVC_Handler() */
#else
#define     smx_ERROR(errnum, h)    smx_EM_SS(errnum, h);  /* switch to SS */
#endif

#define     smx_ERROR_SS(errnum, h) smx_EM(errnum, h);  /* already in SS */

#define     smx_ERROR_EXIT(errnum, exitval, h, id)  /* for use in SSRs */ \
            { \
               smx_ERROR(errnum, h); \
               smx_SSR_EXIT((u32)exitval, id); \
               return(exitval); /* <3> */ \
            }

#define     smx_ERROR_RET(errnum, val, h)  /* for use in functions */ \
            { \
               smx_ERROR(errnum, h); \
               return(val); \
            }

#if SMX_CFG_EVB
#define smx_EVB_LOG_SSR0(id)   smx_EVBLogSSR0(id)
#define smx_EVB_LOG_SSR1(id, p1)   smx_EVBLogSSR1(id, (u32)p1)
#define smx_EVB_LOG_SSR2(id, p1, p2)  smx_EVBLogSSR2(id, (u32)p1, (u32)p2)
#define smx_EVB_LOG_SSR3(id, p1, p2, p3)  smx_EVBLogSSR3(id, (u32)p1, (u32)p2, (u32)p3)
#define smx_EVB_LOG_SSR4(id, p1, p2, p3, p4)   smx_EVBLogSSR4(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4)
#define smx_EVB_LOG_SSR5(id, p1, p2, p3, p4, p5)  smx_EVBLogSSR5(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5)
#define smx_EVB_LOG_SSR6(id, p1, p2, p3, p4, p5, p6)  smx_EVBLogSSR6(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6)
#define smx_EVB_LOG_SSR7(id, p1, p2, p3, p4, p5, p6, p7)  smx_EVBLogSSR7(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6, (u32)p7)
#else
#define smx_EVB_LOG_SSR0(id)
#define smx_EVB_LOG_SSR1(id, p1)
#define smx_EVB_LOG_SSR2(id, p1, p2)
#define smx_EVB_LOG_SSR3(id, p1, p2, p3)
#define smx_EVB_LOG_SSR4(id, p1, p2, p3, p4)
#define smx_EVB_LOG_SSR5(id, p1, p2, p3, p4, p5)
#define smx_EVB_LOG_SSR6(id, p1, p2, p3, p4, p5, p6)
#define smx_EVB_LOG_SSR7(id, p1, p2, p3, p4, p5, p6, p7)
#endif

#if SMX_CFG_MACROS
#define     smx_SSR_ENTER0(id) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR0(id); }
#define     smx_SSR_ENTER1(id, p1) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR1(id, (u32)p1); }
#define     smx_SSR_ENTER2(id, p1, p2) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR2(id, (u32)p1, (u32)p2); }
#define     smx_SSR_ENTER3(id, p1, p2, p3) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR3(id, (u32)p1, (u32)p2, (u32)p3); }
#define     smx_SSR_ENTER4(id, p1, p2, p3, p4) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR4(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4); }
#define     smx_SSR_ENTER5(id, p1, p2, p3, p4, p5) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR5(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5); }
#define     smx_SSR_ENTER6(id, p1, p2, p3, p4, p5, p6) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR6(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6); }
#define     smx_SSR_ENTER7(id, p1, p2, p3, p4, p5, p6, p7) \
                        { smx_srnest++; \
                          smx_ct->err = SMXE_OK; \
                          smx_SAVE_SUSPLOC(); \
                          smx_EVB_LOG_SSR7(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6), (u32)p7; }
#else
#define     smx_SSR_ENTER0(id) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/ \
                          smx_SSREnter(id); }
#define     smx_SSR_ENTER1(id, p1) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/ \
                          smx_SSREnter(id, (u32)p1); }
#define     smx_SSR_ENTER2(id, p1, p2) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/ \
                          smx_SSREnter(id, (u32)p1, (u32)p2); }
#define     smx_SSR_ENTER3(id, p1, p2, p3) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/  \
                          smx_SSREnter(id, (u32)p1, (u32)p2, (u32)p3); }
#define     smx_SSR_ENTER4(id, p1, p2, p3, p4) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/  \
                          smx_SSREnter(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4); }
#define     smx_SSR_ENTER5(id, p1, p2, p3, p4, p5) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/  \
                          smx_SSREnter(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5); }
#define     smx_SSR_ENTER6(id, p1, p2, p3, p4, p5, p6) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/  \
                          smx_SSREnter(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6); }
#define     smx_SSR_ENTER7(id, p1, p2, p3, p4, p5, p6, p7) \
                        { smx_SAVE_SUSPLOC(); /*<7>*/  \
                          smx_SSREnter(id, (u32)p1, (u32)p2, (u32)p3, (u32)p4, (u32)p5, (u32)p6, (u32)p7); }
#endif

#define     smx_SSR_EXIT(ret, id)   smx_SSRExit(ret, id)

/*===========================================================================*
*             Common Functions and Macros for ucode and pcode                *
*                           (to end of file)                                 *
*===========================================================================*/

#ifdef __cplusplus
extern "C" {
#endif

/*===========================================================================*
*                      Functions for Application Use                         *
*===========================================================================*/

void        ainit(u32 par=0);
void        aexit(SMX_ERRNO errnum);
void        smx_EBDisplay(void);
void        smx_EMHook(SMX_ERRNO errnum, void* handle);
void        smx_EMExitHook(SMX_ERRNO errnum, void* handle);
void        smx_EtimeRollover(void);
u32         smx_GetPSR(void);
void        smx_TickISRHook(void);
void        smx_TTHook(void);

#if SMX_CFG_MPU
BOOLEAN     mp_MPACreate(TCB_PTR task, MPA* tmp=NULL, u32 tmsk=MP_TMSK_DFLT, u32 mpasz=MP_MPU_ACTVSZ);
BOOLEAN     mp_MPACreateLSR(LCB_PTR lsr, MPA* tmp=NULL, u32 tmsk=MP_TMSK_DFLT, u32 mpasz=MP_MPU_ACTVSZ);
BOOLEAN     mp_MPASlotMove(u8 dn, u8 sn);
void        mp_MPUInit(void);
void        mp_MPULoad(BOOLEAN task);  /* internal use */
BOOLEAN     mp_MPUSlotLoad(u8 dn, u32* rp);
BOOLEAN     mp_MPUSlotSwap(u8 dn, u32* rp);
u8*         mp_RegionGetHeapR(MPR_PTR rp, u32 sz, u8 sn, u32 attr, const char* name=NULL, u32 hn=0);
u8*         mp_RegionGetHeapT(TCB_PTR task, u32 sz, u8 sn, u32 attr, const char* name=NULL, u32 hn=0);
u8*         mp_RegionGetPoolR(MPR_PTR rp, PCB_PTR pool, u8 sn, u32 attr, const char* name=NULL);
u8*         mp_RegionGetPoolT(TCB_PTR task, PCB_PTR pool, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     mp_RegionMakeR(MPR_PTR rp, u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL);
BOOLEAN     mp_RegionMakeT(u8* bp, u32 sz, u8 sn, u32 attr, const char* name=NULL);
#endif

#ifdef __cplusplus
}
#endif

/*===========================================================================*
*                        Macros for Application Use                          *
*===========================================================================*/

/* Map compiler heap calls to smx heap calls at compile time*/
#if !defined(NO_SMX_HEAP_MACROS)
#define     calloc(n, size)            smx_HeapCalloc(n, size, 0, 0)
#define     free(bp)                   smx_HeapFree(bp, 0)
#define     malloc(num_bytes)          smx_HeapMalloc(num_bytes, 0, 0)
#define     realloc(bp, num_bytes)     smx_HeapRealloc(bp, num_bytes, 0, 0)
#endif

/* Handle table */
#if SMX_CFG_HT
#define     smx_HT_ADD(h, name)        smx_HTAdd(h, name)
#define     smx_HT_DELETE(h)           smx_HTDelete(h)
#else
#define     smx_HT_ADD(h, name)
#define     smx_HT_DELETE(h)
#endif

/* Miscellaneous */
#define     smx_DelayMsec(ms)       smx_TaskSuspend(SMX_CT, smx_ConvMsecToTicks(ms)+1)
#define     smx_DelaySec(s)         smx_TaskSuspend(SMX_CT, ((s)*(SMX_CFG_TICKS_PER_SEC))+1)
#define     smx_DelayTicks(t)       smx_TaskSuspend(SMX_CT, (t)+1)
#define     smx_LSR_INVOKE(lsr, par)   smx_LSRInvokeF(lsr, par)
#if !SMX_CFG_MPU
#define     smx_TaskYield()         smx_TaskBump(smx_ct, SMX_PRI_NOCHG)
#endif


/*===========================================================================*
*                    C++ new() and delete() Replacements                     *
*===========================================================================*/
/*
* These are function prototypes and inline functions for C++ memory
* allocation using global new and delete operators. It is necessary to
* use these rather than the versions provided with the compiler,
* to ensure that smx heap functions are used and that no unwanted behavior
* is present. Also see APP\heap.c for implementation of these
* operators. This code is needed by smx++, PEG+, and possibly other C++
* libraries and application code.
*/

#ifdef __cplusplus

#if defined(__GNUC__) && __EXCEPTIONS && (__cplusplus < 201103L)  /* GNU C++ pre-2011 with exceptions */
#include <cstddef>
#include <new>
void*  newx(size_t s);  /* in heap.c */
typedef int (* PNHX)( size_t );
inline void* operator new(size_t s) throw (std::bad_alloc) { return newx(s); }
inline void operator delete(void* b) throw () { smx_HeapFree(b); }

#elif defined(__CC_ARM) || defined(__GNUC__)  /* ARM, GNU */
void* newx(size_t s);  /* in heap.c */
typedef int (* PNHX)( size_t );
inline void* operator new(size_t s) { return newx(s); }
inline void operator delete(void* b) { smx_HeapFree(b); }

#elif defined(__IAR_SYSTEMS_ICC__)  /* IAR */
void* newx(size_t s);  /* in heap.c */
typedef int (* PNHX)( size_t );
#if !defined(SMX_TXPORT) /* <6> */
void* operator new(size_t s);    /* in heap.c */
void operator delete(void* b);  /* in heap.c */
#endif

#else
#error Define PNHX and new/delete for your compiler.
#endif

PNHX set_new_handler(PNHX a_new_handler);

#endif /* __cplusplus */


/* Notes:
   1. Some compilers report a warning if you do not use the return value
      of macros that return a value. Basically, the compiler is reporting 
      that the statement seems to be useless. The text of the message mentions 
      "side effects" or says the code has no effect.
   2. Macros for SSRs are here rather than xsmx.h so the user can add new SSRs.
   3. return((cast)smx_SSR_EXIT(exitval, id)) is not used in smx_ERROR(),
      since it would be necessary to typecast the return value to whatever
      type the SSR returns. This would necessitate having a different
      version of this macro for each possible SSR return type.
   4. Rounds up. Precision depends on tick rate. See smx Reference Manual.
   5. Rounds to nearest.
   6. Compiler errors that linkage specification is incompatible due to
      ThreadX .h files wrapping around whole includes with extern "C" so this
      whole file gets wrapped including these new/delete operators, causing
      the error. They need to fix their .h files.
   7. smx_SAVE_SUSPLOC() must be in the macro not inside the smx_SSREnter()
      function, because calling it overwrites LR which is what needs to be
      saved.
*/

#endif /* SMX_XAPI_H */

