/*
* xglob.h                                                   Version 5.2.0
*
* smx globals
*
* Copyright (c) 1989-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore
*
*****************************************************************************/

#ifndef SMX_XGLOB_H
#define SMX_XGLOB_H

extern FUNV_PTR      smx_autostop;     /* autostop function address */
extern PCB           smx_bcbs;         /* BCB pool */
extern LCB_PTR       smx_clsr;         /* current LSR */
extern TCB_PTR       smx_ct;           /* current task */
extern BOOLEAN       smx_ctstart;      /* current task start */
#if SMX_CFG_PROFILE
extern CPS           smx_cpa;          /* coarse profile accumulator */
extern CPS           smx_cpd;          /* coarse profile display */
#endif
extern TCB           smx_dcb;          /* dummy control block */
extern BOOLEAN const smx_debug_lib;    /* smx library debug version */
extern BOOLEAN       smx_eben;         /* smx error buffer enabled */
extern EREC_PTR      smx_ebi;          /* first record in error buffer */
extern EREC_PTR      smx_ebn;          /* next record in error buffer */
extern EREC_PTR      smx_ebx;          /* last record in error buffer */
extern PCB           smx_egcbs;        /* EGCB pool */
extern PCB           smx_eqcbs;        /* EQCB pool */
extern u32           smx_errctr;       /* total smx errors */
extern u8            smx_errctrs[SMX_NUM_ERRORS]; /* smx error counters */
extern SMX_ERRNO     smx_errno;        /* last smx error */
extern const char* const smx_errmsg[]; /* error message table */
extern vu32          smx_etime;        /* elapsed time */
#if SMX_CFG_EVB
extern u32           smx_evben;        /* event buffer flags to enable what to log */
extern u32*          smx_evbi;         /* first word in event buffer */
extern u32*          smx_evbn;         /* next word in event buffer */
extern u32*          smx_evbx;         /* last word in event buffer */
#endif
extern void*         smx_freestack;    /* free stack pointer */
extern MUCB_PTR      smx_hmtx[EH_NUM_HEAPS]; /* heap mutex pointer array */
extern BOOLEAN       smx_hmng;         /* run HeapManager */
extern u32           smx_htmo;         /* heap mutex timeout */
extern HTREC_PTR     smx_hti;          /* first record in handle table */
extern HTREC_PTR     smx_htn;          /* next record in handle table */
extern HTREC_PTR     smx_htx;          /* last record in handle table */
extern BOOLEAN       smx_htchg;        /* handle table changed (used by smxAware) */
extern BOOLEAN       smx_htpres;       /* handle table present when TRUE */
extern u32           smx_i_rtc;        /* captured ISR rtc at end of frame */
#if SMX_CFG_STACK_SCAN
extern BOOLEAN       smx_idleup;       /* idle is running at > 0 priority */
#endif
extern BOOLEAN       smx_init;         /* set when smx has been initialized */
extern BOOLEAN       smx_inssu;        /* in smx_StackScanU() */
extern u32           smx_isr_rtc;      /* ISR runtime counter */
extern u32           smx_l_rtc;        /* captured LSR rtc at end of frame */
extern PCB           smx_lcbs;         /* LCB pool */
extern u32           smx_lockctr;      /* scheduler lock nesting counter */
extern LQC_PTR       smx_lqi;          /* pointer to beginning of LSR queue */
extern LQC_PTR       smx_lqx;          /* pointer to end of LSR queue */
extern u32           smx_lqctr;        /* number of LSRs in lq */
#if SMX_CFG_DIAGS
extern u32           smx_lqhwm;        /* LSR queue high water mark */
#endif
extern LQC_PTR       smx_lqin;         /* pointer to next free position in lq */
extern LQC_PTR       smx_lqout;        /* pointer to next LSR to run */
extern u32           smx_lsr_rtc;      /* LSR runtime counter */
extern PCB           smx_mcbs;         /* MCB pool */
#if SMX_CFG_MPU
extern BOOLEAN       smx_mmf_halt;     /* halt on MMF (for debug) */
extern BOOLEAN       smx_mpu_br_off;   /* background region was off in interrupted code */
#endif
extern PCB           smx_mucbs;        /* MUCB pool */
extern const u32     smx_nullcb[];     /* null control block */
extern PCB           smx_pcbs;         /* PCB pool */
extern PCB           smx_picbs;        /* PICB pool */
#if SMX_CFG_PROFILE
extern u32           smx_pidle;        /* % of time idle */
extern u32           smx_povhd;        /* % of time in smx */
extern u32           smx_pwork;        /* % of time doing useful work */
#endif
extern u32           smx_psp_sav;      /* psp stack pointer save */
extern RQCB          smx_rq[SMX_PRI_NUM]; /* ready queue */
extern RQCB_PTR      smx_rqx;             /* ready queue last level */
extern RQCB_PTR      smx_rqtop;           /* pointer to top priority level of smx_rq */
extern u32*          smx_rtcbi;           /* start of smx_rtcb[][] */
extern u32           smx_rtc_frame_ctr;   /* determines rtc frame */
#if SMX_CFG_RTLIM
extern SCB_PTR       smx_rtlimsem;        /* runtime limit gate semaphore */
#endif
#if SMX_CFG_STACK_SCAN
extern void*         smx_scanstack;       /* stack scan pool pointer */
extern void*         smx_scanstack_end;   /* stack scan pool end pointer */
#endif
extern PCB           smx_scbs;         /* SCB pool */
extern SMX_SCHED     smx_sched;        /* scheduler flags */
extern u32*          smx_spmin;        /* start of stack pool */
extern u32           smx_srnest;       /* service routine nesting level */
extern u16           smx_sshwm;        /* ss high water mark */
extern BOOLEAN       smx_sshwmv;       /* ss high water mark valid */
extern u32           smx_ssnest;       /* ss in use and srnest value to switch to ct stack */
#if SMX_CFG_MPU
extern u32           smx_sst[];        /* main system service table */
extern u32*          smx_sstp;         /* curent system service table pointer */
#endif
#if SMX_CFG_MPU && SMX_CFG_SSCTR
extern u32*          smx_sst_ctr;      /* system service call counters */
extern u32           smx_svc_ctr;      /* SVC call counter (total ss calls) */
#endif
extern CBF_PTR       smx_stk_ovfl;     /* pointer to stack overflow handler */
extern vu32          smx_stime;        /* system time */
extern BOOLEAN       smx_stkpl_init;   /* set when the stack pool has been initialized */
extern TCB_PTR       smx_tcbns;        /* next TCB to stack scan*/
extern PCB           smx_tcbs;         /* TCB pool */
extern u32           smx_timeout[SMX_CFG_NUM_TASKS]; /* task timeout array */
extern PCB           smx_tmcbs;        /* TMRCB pool */
extern u32           smx_tmo_indx;     /* index of minimum timeout */
extern u32           smx_tmo_min;      /* minimum waiting timeout value */
extern CB            smx_tqcb;         /* timer queue control block */
extern CB_PTR        smx_tq;           /* timer queue */
extern BOOLEAN       smx_uf_halt;      /* halt on Usage Fault (for debug) */
extern PCB           smx_xcbs;         /* XCB pool */

extern u32           smx_Version;      /* smx version number (consulted by smxAware) */
extern u32           smx_Processor;    /* smx processor version encoding (consulted by smxAware) */

/* System Objects */
extern TCB_PTR       smx_Idle;         /* idle task */
extern LCB_PTR       smx_KeepTimeLSR;
extern LCB_PTR       smx_ProfileLSR;
extern LCB_PTR       smx_TaskDeleteLSR;
extern LCB_PTR       smx_TimeoutLSR;

/* Misc Objects */
extern PCB           spp_obj;          /* smx++ object pool */

/* Pseudo Handles for Event Buffer */
#if SMX_CFG_EVB
extern void*         smx_InvokeH;
extern void*         smx_TickISRH;
#endif

#endif /* SMX_XGLOB_H */

