/*
* app.c                                                     Version 5.2.0
*
* Protosystem application module.
*
* USER: This file should be replaced for your application. See below.
* Add-on demo modules are in the DEMO directory. Demo modules contain
* configuration constants that should be set and directions that you
* should read prior to building demos.
*
* Copyright (c) 1994-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#include "smx.h"
#include "smxmain.h"
#include "acfg.h"

void mcon_init(void);

#ifdef __cplusplus
extern "C" {
#endif

#ifdef FF_DEMO
void ffdemo_init(void);
void ffdemo_exit(void);
#endif

#ifdef __cplusplus
}
#endif


#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = @ ".sys_text"
#endif

#if SMX_CFG_MPU
/*+++++++++++++++++++++++++ FULL DIRECT API (PMODE) ++++++++++++++++++++++++*/
#include "xapip.h"
#endif

/***** APPLICATION INITIALIZATION
*
* This is a typical application initialization function. 
*
* IMPORTANT: Do not call smx functions that wait (i.e. those with a
* timeout parameter). Also, do not call middleware routines, since
* these call smx functions that wait. ainit() (which calls this routine)
* must finish before other tasks run. This function is meant only to
* create the main tasks and objects in your application.
*
*****************************************************************************/

void appl_init(void)
{
   sb_ConWriteString(0,0,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,"SecureSMX Partition Demo 4  (Built " __DATE__ ")");
   sb_MsgDisplay();
   sb_DelayMsec(100);

   mcon_init();

  #if defined(FF_DEMO)
   ffdemo_init();    /* entry to FatFs demo */
  #endif
}


/***** APPLICATION EXIT
* Put application cleanup code here.
*****************************************************************************/

void appl_exit(void)
{
  #if defined(FF_DEMO)
   ffdemo_exit();    /* cleanup for FatFs demo */
  #endif
}
