/*
* reset.c                                                   Version 5.2.0
*
* Reset vector and stack pointer for STM32F7xxxx. Put at start of every
* major code region in case it is biggest and MpuPacker locates it first.
* These must be at offset 0 of the boot flash. They are normally the first
* two entries of the EVT. __low_level_init sets NVIC_VTOR to locate the EVT.
*
* Copyright (c) 2020-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#include "smxbase.h"
#include "bsp.h"

#if defined(__ICCARM__)

#pragma language=extended  /* allow IAR extended syntax in this file */
#pragma segment="CSTACK"

#ifdef __cplusplus
extern "C" {
#endif
void __iar_program_start(void);  /* cmain.s (startup code in IAR dlib) */
#ifdef __cplusplus
}
#endif

/*
   A reset block needs to be created for any code block that could be biggest
   and ordered first by MpuPacker, except sys_code which has the full EVT.
*/

/* Needed since ucom_code is put at start of sys_code (which has full EVT). */
 #ifdef __cplusplus
__root extern const intvec_elem __ucom_reset[] @ ".ucom_reset" =
 #else
__root const intvec_elem __ucom_reset[] @ ".ucom_reset" =
 #endif
{
/*  0 */  { .__ptr = __sfe( "CSTACK" ) },  /* Initial Stack Pointer */
/*  1 */  __iar_program_start,             /* Reset Vector (Entry Point) */
};

#if defined(SMXFS)
 #ifdef __cplusplus
__root extern const intvec_elem __fs_reset[] @ ".fs_reset" =
 #else
__root const intvec_elem __fs_reset[] @ ".fs_reset" =
 #endif
{
/*  0 */  { .__ptr = __sfe( "CSTACK" ) },  /* Initial Stack Pointer */
/*  1 */  __iar_program_start,             /* Reset Vector (Entry Point) */
};
#endif

#if defined(SMXNS)
 #ifdef __cplusplus
__root extern const intvec_elem __ns_reset[] @ ".ns_reset" =
 #else
__root const intvec_elem __ns_reset[] @ ".ns_reset" =
 #endif
{
/*  0 */  { .__ptr = __sfe( "CSTACK" ) },  /* Initial Stack Pointer */
/*  1 */  __iar_program_start,             /* Reset Vector (Entry Point) */
};
#endif

#if defined(SMXUSBD)
 #ifdef __cplusplus
__root extern const intvec_elem __usbd_reset[] @ ".usbd_reset" =
 #else
__root const intvec_elem __usbd_reset[] @ ".usbd_reset" =
 #endif
{
/*  0 */  { .__ptr = __sfe( "CSTACK" ) },  /* Initial Stack Pointer */
/*  1 */  __iar_program_start,             /* Reset Vector (Entry Point) */
};
#endif

#if defined(SMXUSBH)
 #ifdef __cplusplus
__root extern const intvec_elem __usbh_reset[] @ ".usbh_reset" =
 #else
__root const intvec_elem __usbh_reset[] @ ".usbh_reset" =
 #endif
{
/*  0 */  { .__ptr = __sfe( "CSTACK" ) },  /* Initial Stack Pointer */
/*  1 */  __iar_program_start,             /* Reset Vector (Entry Point) */
};
#endif

#else
#error Implement reset.c for your compiler
#endif /* __ICCARM__ */
