/*
* bcon.h                                                    Version 5.2.0
*
* Console display function prototypes and constant declarations.
*
* Copyright (c) 1990-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
* Note: For this API, coordinates are relative to (0,0), so implementations
*       should adjust input parameters as necessary. For example, ANSI
*       terminal coordinates are relative to (1,1), so the implementation
*       should add 1 to each.
*
*****************************************************************************/

#ifndef SB_BCON_H
#define SB_BCON_H

/* definitions */

#define SB_CON_ROWS_MAX      24  /* number of text rows, (1 to 24) */
#define SB_CON_COLS_MAX      80  /* number of text columns, (1 to 80) */
#define SB_CON_BLINK         5   /* blink attribute */

#define SB_CON_SIZE (SB_CON_ROWS_MAX * SB_CON_COLS_MAX)

/*
   ANSI terminal foreground and background colors (ISO 6429)
*/
#define SB_CLR_BLACK        0x00
#define SB_CLR_RED          0x01
#define SB_CLR_GREEN        0x02
#define SB_CLR_YELLOW       0x03
#define SB_CLR_BLUE         0x04
#define SB_CLR_MAGENTA      0x05
#define SB_CLR_CYAN         0x06
#define SB_CLR_WHITE        0x07
/* Map these onto colors above. */
#define SB_CLR_LIGHTRED     0x01
#define SB_CLR_LIGHTGREEN   0x02
#define SB_CLR_BROWN        0x03
#define SB_CLR_LIGHTBLUE    0x04
#define SB_CLR_LIGHTMAGENTA 0x05
#define SB_CLR_LIGHTCYAN    0x06
#define SB_CLR_LIGHTGRAY    0x07
#define SB_CLR_DARKGRAY     0x07


/* API Functions */

#ifdef __cplusplus
extern "C" {
#endif

#if (SB_CON_OUT)
void    sb_ConClearEndOfLine(u32 col, u32 row, u32 len);
void    sb_ConClearEndOfLineUnp(u32 col, u32 row, u32 len);
void    sb_ConClearLine(u32 row);
void    sb_ConClearScreen(void);
void    sb_ConClearScreenUnp(void);
void    sb_ConCursorOff(void);
void    sb_ConCursorOn(void);
BOOLEAN sb_ConDbgMsgMode(void);
void    sb_ConDbgMsgModeSet(BOOLEAN enable);
void    sb_ConInit(u32 port);
u32     sb_ConOutPort(void);
int     sb_ConPutChar(char ch);
int     sb_ConPutString(const char* in_string);
void    sb_ConScroll(void);
void    sb_ConWriteChar(u32 col, u32 row, u32 F_color, u32 B_color, u32 blink, char ch);
void    sb_ConWriteCharUnp(u32 col, u32 row, u32 F_color, u32 B_color, u32 blink, char ch);
void    sb_ConWriteCounter(u32 col, u32 row, u32 F_color, u32 B_color, u32 ctr, u32 radix);
void    sb_ConWriteString(u32 col, u32 row, u32 F_color, u32 B_color, u32 blink, const char* in_string);
void    sb_ConWriteStringUnp(u32 col, u32 row, u32 F_color, u32 B_color, u32 blink, const char* in_string);
void    sb_ConWriteStringNum(u32 col, u32 row, u32 F_color, u32 B_color, u32 blink, const char* in_string, u32 num);
#else
#define sb_ConClearEndOfLine(col, row, len) {}
#define sb_ConClearEndOfLineUnp(col, row, len) {}
#define sb_ConClearLine(row) {}
#define sb_ConClearScreen() {}
#define sb_ConClearScreenUnp() {}
#define sb_ConCursorOff() {}
#define sb_ConCursorOn() {}
#define sb_ConDbgMsgMode() FALSE
#define sb_ConDbgMsgModeSet(enable) {}
#define sb_ConInit(port) {}
#define sb_ConOutPort(void) {}
#define sb_ConPutChar(ch) {}
#define sb_ConPutString(in_string) {}
#define sb_ConScroll() {}
#define sb_ConWriteChar(col, row, F_color, B_color, blink, ch) {}
#define sb_ConWriteCharUnp(col, row, F_color, B_color, blink, ch) {}
#define sb_ConWriteCounter(col, row, F_color, B_color, ctr, radix) {}
#define sb_ConWriteString(col, row, F_color, B_color, blink, in_string) {}
#define sb_ConWriteStringUnp(col, row, F_color, B_color, blink, in_string) {}
#define sb_ConWriteStringNum(col, row, F_color, B_color, blink, in_string, num) {}
#endif

#ifdef __cplusplus
}
#endif

#endif /* SB_BCON_H */

